@@ -1,7 +1,6 @@
 #undef PACKAGE
 #undef VERSION
-#undef USE_PYTHON
-#undef HAVE_PYGTK
+#undef WITH_PYTHON
 #undef USE_XIM
 #undef HAVE_LIBSM
 #undef HAVE_CATGETS
