@@ -1,4 +1,4 @@
-/*  about.h
+/*  Glimmer - about.h
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -21,6 +21,8 @@
 extern "C" {
 #endif
 
+#include "declarations.h"
+
 void do_about(GtkWidget *widget, gpointer data);
 
 #ifdef __cplusplus
