@@ -1,8 +1,33 @@
+/*  Glimmer - bindings.c
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Library General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
+
 #include "declarations.h"
+#include "bindings.h"
+#include "widgets/gdsfile.h"
+#include "build.h"
+#include "edit.h"
+#include "file-io.h"
+#include "main.h"
+#include "misc.h"
+#include "searches.h"
+#include "settings.h"
+#include "syntax-highlight.h"
 #include "macro-language.h"
 #include "commandbar.h"
 #include "windows.h"
-#include "bindings.h"
 
 int get_file_number(void)
 {
@@ -190,30 +215,30 @@ void set_file_done_executing(int file_no)
 int has_selection(void)
 {
    gint value;
-   GtkEditable *editable;
+   GtkExText *text;
    g_return_val_if_fail(cur_file, 0);
-   editable = GTK_EDITABLE(cur_file->text);
-   value = (int)editable->has_selection;
+   text = GTK_EXTEXT(cur_file->text);
+   value = (int)GTK_EXTEXT(cur_file->text)->has_selection;
    return(value);   
 }
 
 int selection_start(void)
 {
    int value;
-   GtkEditable *editable;
+   GtkExText *text;
    g_return_val_if_fail(cur_file, -1);
-   editable = GTK_EDITABLE(cur_file->text);
-   value = (int)editable->selection_start_pos;
+   text = GTK_EXTEXT(cur_file->text);
+   value = (int)text->selection_start_pos;
    return(value);
 }
 
 int selection_end(void)
 {
    int value;
-   GtkEditable *editable;
+   GtkExText *text;
    g_return_val_if_fail(cur_file, -1);
-   editable = GTK_EDITABLE(cur_file->text);
-   value = (int)editable->selection_end_pos;
+   text = GTK_EXTEXT(cur_file->text);
+   value = (int)text->selection_end_pos;
    return(value);
 }
 
@@ -221,23 +246,23 @@ void move(int relative_movement)
 {
    int index;
    g_return_if_fail(cur_file);
-   index = gtk_extext_get_position(GTK_EDITABLE(cur_file->text));
+   index = gtk_extext_get_position(GTK_EXTEXT(cur_file->text));
    index += relative_movement;
    g_return_if_fail(index >= 0);
-   gtk_extext_set_position(GTK_EDITABLE(cur_file->text), index);
+   gtk_extext_set_position(GTK_EXTEXT(cur_file->text), index);
 }
 
 void move_to(int index)
 {
    g_return_if_fail(cur_file);
    g_return_if_fail(index >= 0);
-   gtk_extext_set_position(GTK_EDITABLE(cur_file->text), index);
+   gtk_extext_set_position(GTK_EXTEXT(cur_file->text), index);
 }
 
 int current_position(void)
 {
    g_return_val_if_fail(cur_file, -1);
-   return gtk_extext_get_position(GTK_EDITABLE(cur_file->text));
+   return gtk_extext_get_position(GTK_EXTEXT(cur_file->text));
 }
 
 void move_to_line (int index)
@@ -258,7 +283,7 @@ void move_lines(int relative_movement)
 int current_line(void)
 {
    g_return_val_if_fail(cur_file, -1);
-   return GTK_EXTEXT(cur_file->text)->line_pos_index + 1;
+   return GTK_EXTEXT(cur_file->text)->line_number + 1;
 }
 
 int line_start(void)
@@ -268,7 +293,7 @@ int line_start(void)
    gint start = 0;
    g_return_val_if_fail(cur_file, -1);
    text = GTK_EXTEXT(cur_file->text);
-   linedata = gtk_extext_get_line_data(text, text->line_pos_index, text->scroll_line_start);
+   linedata = gtk_extext_get_line_data(text, text->line_number, text->scroll_line_start);
    if(linedata) start = linedata->startpos;
    g_free(linedata);
    return(start);
@@ -282,7 +307,7 @@ int line_end(void)
    g_return_val_if_fail(cur_file, -1);
    text = GTK_EXTEXT(cur_file->text);
    end = text->length;
-   linedata = gtk_extext_get_line_data(text, text->line_pos_index, text->scroll_line_start);
+   linedata = gtk_extext_get_line_data(text, text->line_number, text->scroll_line_start);
    if(linedata) end = linedata->endpos;
    g_free(linedata);
    return(end);
@@ -310,7 +335,7 @@ int line_by_pos(int pos)
 {
    GtkExTextLineData *linedata;
    g_return_val_if_fail(cur_file, -1);
-   linedata = gtk_extext_get_line_by_char_pos(GTK_EXTEXT(cur_file->text), pos);
+   linedata = gtk_extext_get_line_by_char_pos(GTK_EXTEXT(cur_file->text), pos, NULL);
    pos = linedata ? linedata->line_number+1 : -1;
    return(pos);
 }
@@ -325,14 +350,14 @@ void move_to_col(int index)
 char* get_text(int begin, int end)
 {
    g_return_val_if_fail(cur_file, NULL);
-   return gtk_extext_get_text(GTK_EXTEXT(cur_file->text),begin,end);
+   return gtk_extext_get_chars(GTK_EXTEXT(cur_file->text), begin, end);
 }
 
 char get_current_char(void)
 {
    int index;
    g_return_val_if_fail(cur_file, -1);
-   index = gtk_extext_get_position(GTK_EDITABLE(cur_file->text));
+   index = gtk_extext_get_position(GTK_EXTEXT(cur_file->text));
    return GTK_EXTEXT_INDEX(GTK_EXTEXT(cur_file->text), index);
 }
 
@@ -341,10 +366,10 @@ char get_next_char(void)
    char c;
    int index;
    g_return_val_if_fail(cur_file, -1);
-   index = gtk_extext_get_position(GTK_EDITABLE(cur_file->text));
+   index = gtk_extext_get_position(GTK_EXTEXT(cur_file->text));
    c = GTK_EXTEXT_INDEX(GTK_EXTEXT(cur_file->text),index);
    index++;
-   gtk_extext_set_position(GTK_EDITABLE(cur_file->text), index);
+   gtk_extext_set_position(GTK_EXTEXT(cur_file->text), index);
    return(c);
 }
 
@@ -368,11 +393,11 @@ void insert_in(int file_no, int pos, char* text)
 void insert(char* text)
 {
    GdsEditor *editor;
-   GtkEditable *editable;
+   GtkExText *extext;
    g_return_if_fail(text != NULL);
    editor = GDS_EDITOR(cur_file->text);
-   editable = GTK_EDITABLE(cur_file->text);
-   gds_editor_insert_text(editor, text, strlen(text), (gint *)&editable->current_pos);
+   extext = GTK_EXTEXT(cur_file->text);
+   gds_editor_insert_text(editor, text, strlen(text), (gint *)&extext->current_pos);
 }
 
 void delete_text(int start, int end)
@@ -491,8 +516,8 @@ void clear_entry(void)
 void select_area(int from, int to)
 {
    g_return_if_fail(cur_file);
-   gtk_extext_set_position(GTK_EDITABLE(cur_file->text), to);
-   gtk_editable_select_region(GTK_EDITABLE(cur_file->text), from, to);
+   gtk_extext_set_position(GTK_EXTEXT(cur_file->text), to);
+   gtk_extext_select_region(GTK_EXTEXT(cur_file->text), from, to);
    gtk_widget_grab_focus(cur_file->text);
 }
 
@@ -501,8 +526,8 @@ void select_text(int file_no, int from, int to)
    GdsFile *file;
    file = (GdsFile *)g_list_nth_data(files_list, file_no);
    g_return_if_fail(file);
-   gtk_extext_set_position(GTK_EDITABLE(cur_file->text), to);
-   gtk_editable_select_region(GTK_EDITABLE(file->text), from, to);
+   gtk_extext_set_position(GTK_EXTEXT(cur_file->text), to);
+   gtk_extext_select_region(GTK_EXTEXT(file->text), from, to);
    gtk_widget_grab_focus(cur_file->text);
 }
 
@@ -574,19 +599,21 @@ int find_in(int file_no, char *text)
 {
    int success = FALSE;
    GtkExTextMatch m;
-   GtkEditable *editable;
+   GtkExText *extext;
    GdsFile *file;
    file = (GdsFile *)g_list_nth_data(files_list, file_no);
-	g_return_val_if_fail(cur_file, FALSE);
-   editable = GTK_EDITABLE(file->text);
-   success = (int)find_text(GTK_EXTEXT(cur_file->text), editable->current_pos, GTK_EXTEXT(cur_file->text)->length, text, FALSE, FALSE, TRUE, &m);
+   g_return_val_if_fail(cur_file, FALSE);
+   extext = GTK_EXTEXT(file->text);
+   success = (int)find_text(extext, extext->current_pos, extext->length, text, FALSE, FALSE, TRUE, &m);
+#ifdef WITH_PYTHON
    gtk_widget_grab_focus(macro_entry);
+#endif
    return(success);
 }
 
 int find_regex(char *text)
 {
-	g_return_val_if_fail(cur_file, FALSE);
+   g_return_val_if_fail(cur_file, FALSE);
    return(find_regex_in(get_file_number(), text));
 }
 
@@ -594,19 +621,21 @@ int find_regex_in(int file_no, char *text)
 {
    int success = FALSE;
    GtkExTextMatch m;
-   GtkEditable *editable;
+   GtkExText *extext;
    GdsFile *file;
    file = (GdsFile *)g_list_nth_data(files_list, file_no);
-	g_return_val_if_fail(cur_file, FALSE);
-   editable = GTK_EDITABLE(file->text);
-   success = (int)find_text(GTK_EXTEXT(cur_file->text), editable->current_pos, GTK_EXTEXT(cur_file->text)->length, text, FALSE, TRUE, FALSE, &m);
+   g_return_val_if_fail(cur_file, FALSE);
+   extext = GTK_EXTEXT(file->text);
+   success = (int)find_text(extext, extext->current_pos, extext->length, text, FALSE, TRUE, FALSE, &m);
+#ifdef WITH_PYTHON
    gtk_widget_grab_focus(macro_entry);
+#endif
    return(success);
 }
 
 int replace(char *search_text, char *replacer)
 {
-	g_return_val_if_fail(cur_file, FALSE);
+   g_return_val_if_fail(cur_file, FALSE);
    return(replace_in(get_file_number(), search_text, replacer));
 }
 
@@ -614,23 +643,25 @@ int replace_in(int file_no, char *search_text, char *replacer)
 {
    int success = FALSE;
    GtkExTextMatch m;
-   GtkEditable *editable;
+   GtkExText *text;
    GdsFile *file;
    file = (GdsFile *)g_list_nth_data(files_list, file_no);
 	g_return_val_if_fail(cur_file, FALSE);
-   editable = GTK_EDITABLE(file->text);
-   if(find_text(GTK_EXTEXT(file->text), editable->current_pos, GTK_EXTEXT(file->text)->length, search_text, TRUE, FALSE, TRUE, &m))
+   text = GTK_EXTEXT(file->text);
+   if(find_text(text, text->current_pos, text->length, search_text, TRUE, FALSE, TRUE, &m))
    {
-      replace_text(GTK_EXTEXT(file->text), replacer);
+      replace_text(text, replacer);
       success = TRUE;
    }
+#ifdef WITH_PYTHON
    gtk_widget_grab_focus(macro_entry);
+#endif
    return(success);
 }
 
 int replace_all(char *search_text, char *replacer)
 {
-	g_return_val_if_fail(cur_file, FALSE);
+   g_return_val_if_fail(cur_file, FALSE);
    return(replace_all_in(get_file_number(), search_text, replacer));
 }
 
@@ -639,8 +670,8 @@ int replace_all_in(int file_no, char *search_text, char *replacer)
    int success = FALSE;
    GdsFile *file;
    file = (GdsFile *)g_list_nth_data(files_list, file_no);
-	g_return_val_if_fail(cur_file, FALSE);
-	success = (int)replace_all_text(GTK_EXTEXT(file->text), search_text, replacer, TRUE, FALSE, GTK_EXTEXT(file->text)->length);
+   g_return_val_if_fail(cur_file, FALSE);
+   success = (int)replace_all_text(GTK_EXTEXT(file->text), search_text, replacer, TRUE, FALSE, GTK_EXTEXT(file->text)->length);
    return(success);
 }
 
