@@ -1,4 +1,4 @@
-/*  build.h
+/*  Glimmer - build.h
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -14,13 +14,16 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ifndef BUILD_H
-#define BUILD_H
+#ifndef _BUILD_H_
+#define _BUILD_H_
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+#include "declarations.h"
+#include "widgets/gdsfile.h"
+
 void set_build_sensitivity(void);
 void compile_file_cb(GtkWidget *widget, gpointer data);
 void compile_file_full(GtkWidget *widget, GdsFile *file);
