@@ -1,4 +1,4 @@
-/*  color-settings.h
+/*  Glimmer - color-settings.h
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -14,13 +14,14 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ifndef _COLOR_SETTINGS__
-#define _COLOR_SETTINGS__
+#ifndef _COLOR_SETTINGS_H_
+#define _COLOR_SETTINGS_H_
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+#include "declarations.h"
 #include "syntax-highlight.h"
 
 void change_colors_cb(GtkWidget *widget, gpointer data);
