@@ -1,4 +1,4 @@
-/*  commandbar.h
+/*  Glimmer - commandbar.h
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -14,13 +14,17 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ifndef _COMMAND_BAR_H_
-#define _COMMAND_BAR_H_
+#ifndef _COMMANDBAR_H_
+#define _COMMANDBAR_H_
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+#ifdef WITH_PYTHON
+#include "declarations.h"
+#include "macro-language.h"
+
 void make_command_combo(void);
 void command_combo_set_history(GList *history);
 gboolean command_combo_add_string(gchar *string);
@@ -28,6 +32,7 @@ gboolean command_combo_add_string(gchar *string);
 extern GtkWidget *commandbar;
 extern GtkWidget *macro_combo;
 extern GtkWidget *macro_entry;
+#endif
 
 #ifdef __cplusplus
 }
