@@ -1,7 +1,33 @@
-#include <unistd.h>
+/*  Glimer - file-io.c
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Library General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
+
 #include "declarations.h"
+#include "file-io.h"
+#include "dialogs.h"
+#include "main.h"
+#include "misc.h"
 #include "macro-language.h"
+#include "mdi-routines.h"
+#include "menus.h"
+#include "properties.h"
+#include "settings.h"
+#include "session.h"
 #include "signals.h"
+#include "syntax-highlight.h"
+#include "widgets/gdstearaway.h"
 
 gchar *last_dir = NULL;
 GList *recent_files = NULL;
@@ -138,7 +164,9 @@ void open_file_real(gchar *filename)
    GDS_EDITOR(cur_file->text)->changed = FALSE;
    cur_file->modtime = get_time(cur_file->filename);
    adjust_sensitivity();
+#ifdef WITH_PYTHON
    adjust_popup_menu();
+#endif
    display_message("File opened.", FLASH);
    if(general_preferences.auto_save) save_session_cb(NULL, NULL);
 }
@@ -147,7 +175,7 @@ void revert_current_file(GtkWidget *widget, gpointer data)
 {
    if(check_if_file_exists(cur_file->filename) && gtk_extext_get_editable(GTK_EXTEXT(cur_file->text)))
    {
-      gtk_editable_delete_text(GTK_EDITABLE(cur_file->text), 0, gtk_extext_get_length(GTK_EXTEXT(cur_file->text)));
+      gtk_extext_delete_text(GTK_EXTEXT(cur_file->text), 0, gtk_extext_get_length(GTK_EXTEXT(cur_file->text)));
       open_file_add_text(cur_file->text, cur_file->filename);
       GDS_EDITOR(cur_file->text)->changed = FALSE;
       cur_file->modtime = get_time(cur_file->filename);
@@ -155,14 +183,16 @@ void revert_current_file(GtkWidget *widget, gpointer data)
    }
    else if(strstr(cur_file->filename, "Untitled"))
    {
-      gtk_editable_delete_text(GTK_EDITABLE(cur_file->text), 0, gtk_extext_get_length(GTK_EXTEXT(cur_file->text)));
+      gtk_extext_delete_text(GTK_EXTEXT(cur_file->text), 0, gtk_extext_get_length(GTK_EXTEXT(cur_file->text)));
       GDS_EDITOR(cur_file->text)->changed = FALSE;
       cur_file->modtime = get_time(cur_file->filename);
       display_message("File reverted to previous state.", FLASH);
    }
    gtk_signal_emit_by_name(GTK_OBJECT(cur_file->text), "move_to_column", 0, NULL);
    adjust_sensitivity();
+#ifdef WITH_PYTHON
    app_emit_scripting_signal("revert-file");
+#endif
 }
 
 void save_file_cb(GtkWidget *widget, gpointer data)
@@ -194,7 +224,7 @@ void save_file_real(GdsFile *gds_file)
       save_error(gds_file->filename);
       return;   
    }
-   all_text = gtk_editable_get_chars(GTK_EDITABLE(gds_file->text), 0, gtk_extext_get_length(GTK_EXTEXT(gds_file->text)));
+   all_text = gtk_extext_get_chars(GTK_EXTEXT(gds_file->text), 0, gtk_extext_get_length(GTK_EXTEXT(gds_file->text)));
    if(gtk_extext_get_length(GTK_EXTEXT(gds_file->text))) fprintf(file, "%s", all_text);
    fclose(file);
    if(all_text) g_free(all_text);
@@ -203,7 +233,9 @@ void save_file_real(GdsFile *gds_file)
    gtk_signal_emit_by_name(GTK_OBJECT(gds_file->text), "move_to_column", 0, NULL);
    back_file = cur_file;
    cur_file = gds_file;
+#ifdef WITH_PYTHON
    app_emit_scripting_signal("save-file");
+#endif
    cur_file = back_file;
    display_message("File Saved.", FLASH);
 }
@@ -277,7 +309,7 @@ void save_file_as_real(GdsFile *gds_file, gchar *filename)
       strcpy(last_dir, temp);
    }
    g_free(temp);
-   all_text = gtk_editable_get_chars(GTK_EDITABLE(gds_file->text), 0, gtk_extext_get_length(GTK_EXTEXT(gds_file->text)));
+   all_text = gtk_extext_get_chars(GTK_EXTEXT(gds_file->text), 0, gtk_extext_get_length(GTK_EXTEXT(gds_file->text)));
    if(all_text) fprintf(file, "%s", all_text);
    fclose(file);
    temp = get_path_from_filename(cur_file->filename);
@@ -310,7 +342,9 @@ void save_file_as_real(GdsFile *gds_file, gchar *filename)
    set_window_title();
    back_file = cur_file;
    cur_file = gds_file;
+#ifdef WITH_PYTHON
    app_emit_scripting_signal("save-as-file");
+#endif
    cur_file = back_file;
    display_message("File saved with new name.", FLASH);
    if(general_preferences.auto_save) save_session_cb(NULL, NULL);
@@ -326,13 +360,19 @@ void save_all_files_cb(GtkWidget *widget, gpointer data)
    {
       file = (GdsFile *)g_list_nth_data(files_list, i);
       save_file_or_dialog(file);
+      if(!GDS_EDITOR(file->text)->changed && file->changed_set)
+      {
+         file->changed_set = FALSE;
+         gds_file_set_title(file, file->filename, general_preferences.full_filenames);
+         gtk_signal_connect(GTK_OBJECT(file->menu_item), "activate", GTK_SIGNAL_FUNC(gds_file_focus_with_raise), file);
+      }
    }
    display_message("All files saved.", FLASH);   
 }
 
 void reload_file_cb(GtkWidget *widget, GdsFile *gds_file)
 {
-   gtk_editable_delete_text(GTK_EDITABLE(gds_file->text), 0, GTK_EXTEXT(gds_file->text)->length);
+   gtk_extext_delete_text(GTK_EXTEXT(gds_file->text), 0, GTK_EXTEXT(gds_file->text)->length);
 	open_file_add_text(gds_file->text, gds_file->filename);
 	GDS_EDITOR(gds_file->text)->changed = FALSE;
    gds_file->modtime = get_time(gds_file->filename);
@@ -381,7 +421,7 @@ void save_file_or_dialog(GdsFile *gds_file)
       save_error(gds_file->filename);
       return;   
    }
-   all_text = gtk_editable_get_chars(GTK_EDITABLE(gds_file->text), 0, gtk_extext_get_length(GTK_EXTEXT(gds_file->text)));
+   all_text = gtk_extext_get_chars(GTK_EXTEXT(gds_file->text), 0, gtk_extext_get_length(GTK_EXTEXT(gds_file->text)));
    if(gtk_extext_get_length(GTK_EXTEXT(gds_file->text))) fprintf(file, "%s", all_text);
    fclose(file);
    if(all_text) g_free(all_text);
@@ -390,7 +430,9 @@ void save_file_or_dialog(GdsFile *gds_file)
    gtk_signal_emit_by_name(GTK_OBJECT(gds_file->text), "move_to_column", 0, NULL);
    back_file = cur_file;
    cur_file = gds_file;
+#ifdef WITH_PYTHON
    app_emit_scripting_signal("save-file");
+#endif
    cur_file = back_file;
    display_message("File Saved.", FLASH);
 }
@@ -428,7 +470,9 @@ void close_file_real(GdsFile *file)
    g_return_if_fail(file != NULL);
    g_return_if_fail(GDS_IS_FILE(file));
    if(GDS_TEAR_AWAY(file->tear_away)->torn) gds_tear_away_bust_a_switch(GDS_TEAR_AWAY(file->tear_away));
+#ifdef WITH_PYTHON
    app_emit_scripting_signal("close-file");
+#endif
    detach_highlight_tables(file);
    files_list = g_list_remove(files_list, file);
    total_files--;
@@ -471,7 +515,7 @@ gboolean open_file_add_text(GtkWidget *text, gchar *filename)
    gpointer buffer;
    gint bytes;
    gint pos;
-   pos = gtk_extext_get_position(GTK_EDITABLE(text));
+   pos = gtk_extext_get_position(GTK_EXTEXT(text));
    
    if(!(file = fopen(filename, "rb")))
    {
@@ -481,12 +525,12 @@ gboolean open_file_add_text(GtkWidget *text, gchar *filename)
    buffer = g_new(char, BUF_SIZE);
    while((bytes = fread(buffer, sizeof(char), BUF_SIZE, file)))
    {
-      gtk_editable_insert_text(GTK_EDITABLE(text), (gchar *)buffer, bytes, (gint *)&GTK_EDITABLE(text)->current_pos);
+      gtk_extext_insert_text(GTK_EXTEXT(text), (gchar *)buffer, bytes, (gint *)&GTK_EXTEXT(text)->current_pos);
       if(bytes < BUF_SIZE) break;
    }
    g_free(buffer);   
    fclose(file);
-   gtk_extext_set_position(GTK_EDITABLE(text), pos);
+   gtk_extext_set_position(GTK_EXTEXT(text), pos);
    gtk_extext_thaw(GTK_EXTEXT(text));
    gtk_widget_grab_focus(text);
    return TRUE;
@@ -555,100 +599,6 @@ void file_create(GtkWidget *widget, gchar *filename)
       fclose(file);
 }
 
-GList *build_file_list(void)
-{
-   GList *temp;
-   temp = g_list_copy(files_list);
-   return(temp);
-}
-
-void setup_recent_files_list(gint max_files)
-{
-   GList *temp = NULL;
-   GList *current = NULL;
-   temp = build_glist_from_file("files.hist", max_files);
-   build_recent_files_menu(temp);
-   
-   current = g_list_first(temp);
-   while(current)
-   {
-      g_free(current->data);
-      current = current->next;
-   }
-   g_list_free(temp);
-}
-
-void build_recent_files_menu(GList *file_strings)
-{
-   GList *current = NULL;
-   gchar *temp = NULL;
-   GtkWidget *menuitem;
-   
-   kill_old_recent_menu();
-   
-   current = g_list_first(file_strings);
-   while(current)
-   {
-      temp = (gchar *)current->data;
-      menuitem = gtk_menu_item_new_with_label(temp);
-      recent_files = g_list_append(recent_files, (gpointer)menuitem);
-      gtk_signal_connect(GTK_OBJECT(menuitem), "activate", GTK_SIGNAL_FUNC(recent_files_callback), 0);  
-      gtk_menu_append(GTK_MENU(recentmenu), menuitem);
-      gtk_widget_show(menuitem);
-      current = current->next;
-   }
-}
-
-void recent_files_callback(GtkWidget *widget, gpointer data)
-{
-   GtkWidget *accel;
-   gchar *string;
-   g_return_if_fail(GTK_IS_BIN(widget));
-   
-   accel = GTK_BIN(widget)->child;
-   gtk_label_get(GTK_LABEL(accel), &string);
-
-   if(check_if_file_exists(string))
-   {
-      open_file_real(string);
-   }
-   else
-   {
-      if(check_if_file_can_exist(string))
-      {
-         remove(string);
-         file_dne_error(string, GTK_SIGNAL_FUNC(open_file_error_cb));
-      }
-      else
-      {
-         read_error(string);
-      }
-   }
-}
-
-void add_file_to_recent_list(gchar *filename)
-{
-   add_string_to_file("files.hist", filename);
-   setup_recent_files_list(general_preferences.history);
-}
-
-void kill_old_recent_menu(void)
-{
-   GList *temp = NULL;
-   GtkWidget *current;
-   
-   temp = g_list_first(recent_files);
-   while(temp)
-   {
-      current = (GtkWidget *)temp->data;
-      if(GTK_IS_WIDGET(current))
-         gtk_widget_destroy(current);
-      temp = temp->next;
-   }
-   g_list_free(recent_files);
-   recent_files = NULL;
-}
-
 gint get_time(gchar *filename)
 {
    struct stat st;
