@@ -1,4 +1,4 @@
-/*  findbar.h
+/*  Glimmer - findbar.h
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -14,13 +14,15 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ifndef _FIND_BAR_H_
-#define _FIND_BAR_H_
+#ifndef _FINDBAR_H_
+#define _FINDBAR_H_
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+#include "declarations.h"
+
 void make_find_toolbar(void);
 void execute_find_cb(GtkWidget* widget, gpointer data);
 void find_combo_set_history(GList *history);
