@@ -1,5 +1,28 @@
+/*  Glimmer - getprefs.c
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Library General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
+
 #include "declarations.h"
+#include "getprefs.h"
 #include "colors.h"
+#include "color-settings.h"
+#include "main.h"
+#include "searches.h"
+#include "settings.h"
+#include "session.h"
+#include "syntax-highlight.h"
 
 void get_prefs(void)
 {
@@ -54,25 +77,25 @@ void get_prefs(void)
    }
 
    text = gnome_config_get_string("/" PACKAGE "/Fonts/font_name");
-   if(text && strlen(text) > 1)
+   if(text && strlen(text) > 0)
    {
       strcpy(general_preferences.font_name, text);
       g_free(text);
    }
    text = gnome_config_get_string("/" PACKAGE "/Fonts/italic_name");
-   if(text && strlen(text) > 1)
+   if(text && strlen(text) > 0)
    {
       strcpy(general_preferences.italic_name, text);
       g_free(text);
    }
    text = gnome_config_get_string("/" PACKAGE "/Fonts/bold_name");
-   if(text && strlen(text) > 1)
+   if(text && strlen(text) > 0)
    {
       strcpy(general_preferences.bold_name, text);
       g_free(text);
    }
    text = gnome_config_get_string("/" PACKAGE "/Fonts/boldi_name");
-   if(text && strlen(text) > 1)
+   if(text && strlen(text) > 0)
    {
       strcpy(general_preferences.boldi_name, text);
       g_free(text);
@@ -90,9 +113,12 @@ void get_prefs(void)
    general_preferences.bm_bg.red = gnome_config_get_int("/" PACKAGE "/Colors/bm_bg.red");
    general_preferences.bm_bg.green = gnome_config_get_int("/" PACKAGE "/Colors/bm_bg.green");
    general_preferences.bm_bg.blue = gnome_config_get_int("/" PACKAGE "/Colors/bm_bg.blue");
+   general_preferences.gutter_bg.red = gnome_config_get_int("/" PACKAGE "/Colors/gutter_bg.red");
+   general_preferences.gutter_bg.green = gnome_config_get_int("/" PACKAGE "/Colors/gutter_bg.green");
+   general_preferences.gutter_bg.blue = gnome_config_get_int("/" PACKAGE "/Colors/gutter_bg.blue");
 
    text = gnome_config_get_string("/" PACKAGE "/Background/background");
-   if(text && strlen(text) > 1)
+   if(text && strlen(text) > 0)
    {
       strcpy(general_preferences.pixmap_name, text);
       g_free(text);
@@ -120,6 +146,19 @@ void get_prefs(void)
    general_preferences.w = gnome_config_get_int("/" PACKAGE "/Geometry/w");
    general_preferences.h = gnome_config_get_int("/" PACKAGE "/Geometry/h");
 
+   text = gnome_config_get_string("/" PACKAGE "/Print Settings/print_font");
+   if(text && strlen(text) > 0)
+   {
+      strcpy(general_preferences.print_font, text);
+      g_free(text);
+   }
+
+   general_preferences.print_landscape = gnome_config_get_int("/" PACKAGE "/Print Settings/print_landscape=0");
+   general_preferences.print_header = gnome_config_get_int("/" PACKAGE "/Print Settings/print_header=1");
+   general_preferences.print_with_wrap = gnome_config_get_int("/" PACKAGE "/Print Settings/print_with_wrap=0");
+   general_preferences.print_numbers = gnome_config_get_int("/" PACKAGE "/Print Settings/print_numbers=1");
+   general_preferences.print_every_n_lines = gnome_config_get_int("/" PACKAGE "/Print Settings/print_every_n_lines=1");
+
    case_sensitive = gnome_config_get_int("/" PACKAGE "/Find-Replace/case_sensitive=0");
    search_style = gnome_config_get_int("/" PACKAGE "/Find-Replace/search_style=0");
    search_start = gnome_config_get_int("/" PACKAGE "/Find-Replace/search_start=0");
@@ -173,6 +212,10 @@ void save_prefs(void)
    gnome_config_set_int("/" PACKAGE "/Colors/bm_bg.red", general_preferences.bm_bg.red);
    gnome_config_set_int("/" PACKAGE "/Colors/bm_bg.green", general_preferences.bm_bg.green);
    gnome_config_set_int("/" PACKAGE "/Colors/bm_bg.blue", general_preferences.bm_bg.blue);
+   gnome_config_set_int("/" PACKAGE "/Colors/gutter_bg.red", general_preferences.gutter_bg.red);
+   gnome_config_set_int("/" PACKAGE "/Colors/gutter_bg.green", general_preferences.gutter_bg.green);
+   gnome_config_set_int("/" PACKAGE "/Colors/gutter_bg.blue", general_preferences.gutter_bg.blue);
+
    text = general_preferences.pixmap_name;
    gnome_config_set_string("/" PACKAGE "/Background/background", text);
 
@@ -197,6 +240,14 @@ void save_prefs(void)
    gnome_config_set_int("/" PACKAGE "/Geometry/y", general_preferences.y);
    gnome_config_set_int("/" PACKAGE "/Geometry/w", general_preferences.w);
    gnome_config_set_int("/" PACKAGE "/Geometry/h", general_preferences.h);
+
+   text = general_preferences.print_font;
+   gnome_config_set_string("/" PACKAGE "/Print Settings/print_font", text);
+   gnome_config_set_int("/" PACKAGE "/Print Settings/print_landscape", general_preferences.print_landscape);
+   gnome_config_set_int("/" PACKAGE "/Print Settings/print_header", general_preferences.print_header);
+   gnome_config_set_int("/" PACKAGE "/Print Settings/print_with_wrap", general_preferences.print_with_wrap);
+   gnome_config_set_int("/" PACKAGE "/Print Settings/print_numbers", general_preferences.print_numbers);
+   gnome_config_set_int("/" PACKAGE "/Print Settings/print_every_n_lines", general_preferences.print_every_n_lines);
    gnome_config_sync();
 }
 
@@ -246,6 +297,9 @@ void save_defaults(void)
    gnome_config_set_int("/" PACKAGE "/Colors/bm_bg.red", default_preferences.bm_bg.red);
    gnome_config_set_int("/" PACKAGE "/Colors/bm_bg.green", default_preferences.bm_bg.green);
    gnome_config_set_int("/" PACKAGE "/Colors/bm_bg.blue", default_preferences.bm_bg.blue);
+   gnome_config_set_int("/" PACKAGE "/Colors/gutter_bg.red", default_preferences.gutter_bg.red);
+   gnome_config_set_int("/" PACKAGE "/Colors/gutter_bg.green", default_preferences.gutter_bg.green);
+   gnome_config_set_int("/" PACKAGE "/Colors/gutter_bg.blue", default_preferences.gutter_bg.blue);
 
    text = default_preferences.pixmap_name;
    gnome_config_set_string("/" PACKAGE "/Background/background", text);
@@ -271,6 +325,14 @@ void save_defaults(void)
    gnome_config_set_int("/" PACKAGE "/Geometry/y", default_preferences.y);
    gnome_config_set_int("/" PACKAGE "/Geometry/w", default_preferences.w);
    gnome_config_set_int("/" PACKAGE "/Geometry/h", default_preferences.h);
+
+   text = default_preferences.print_font;
+   gnome_config_set_string("/" PACKAGE "/Print Settings/print_font", text);
+   gnome_config_set_int("/" PACKAGE "/Print Settings/print_landscape", default_preferences.print_landscape);
+   gnome_config_set_int("/" PACKAGE "/Print Settings/print_header", default_preferences.print_header);
+   gnome_config_set_int("/" PACKAGE "/Print Settings/print_with_wrap", default_preferences.print_with_wrap);
+   gnome_config_set_int("/" PACKAGE "/Print Settings/print_numbers", default_preferences.print_numbers);
+   gnome_config_set_int("/" PACKAGE "/Print Settings/print_every_n_lines", default_preferences.print_every_n_lines);
    gnome_config_sync();
 }
 
@@ -325,8 +387,9 @@ void build_defaults(void)
    style = text_widget->style;
    default_preferences.fg = general_preferences.fg = style->text[GTK_STATE_NORMAL];
    default_preferences.bg = general_preferences.bg = style->bg[GTK_STATE_NORMAL];
-   default_preferences.bm_bg = general_preferences.bm_bg = style->bg[GTK_STATE_PRELIGHT];
    default_preferences.s_bg = general_preferences.s_bg = style->bg[GTK_STATE_SELECTED];
+   default_preferences.bm_bg = general_preferences.bm_bg = style->bg[GTK_STATE_PRELIGHT];
+   default_preferences.gutter_bg = general_preferences.gutter_bg = style->bg[GTK_STATE_INSENSITIVE];
    DefaultCache.colors[DEFAULT] = HighlightCache.colors[DEFAULT] = default_preferences.fg;
    gtk_widget_destroy(win);
    default_preferences.bg_pixmap = general_preferences.bg_pixmap = NULL;
@@ -353,6 +416,13 @@ void build_defaults(void)
    default_preferences.y = 0;
    default_preferences.w = 750;
    default_preferences.h = 450;
+   strcpy(default_preferences.print_font, "Courier 12.000000");
+   strcpy(general_preferences.print_font, default_preferences.print_font);
+   default_preferences.print_landscape = 0;
+   default_preferences.print_header = 1;
+   default_preferences.print_with_wrap = 0;
+   default_preferences.print_numbers = 1;
+   default_preferences.print_every_n_lines = 1;
 }
 
 void get_saved_colors(void)
@@ -383,6 +453,7 @@ void get_saved_colors(void)
          HighlightCache.types[i] = info->type;
          HighlightCache.flags[i] = info->flags;
          g_free(info);
+         g_free(text); /* Memprof claims there is a leak here. This line should fix it. */
       }
    }
 }
@@ -410,10 +481,10 @@ void save_colors(void)
       if(HighlightCache.types[i] == FONT_BOLD || HighlightCache.types[i] == FONT_BOLD_ITALIC)
          text[count++] = 'b';
       text[count] = '\0';
-      if(HighlightCache.flags[i] & STYLE_UNDER)
+      if(HighlightCache.flags[i] & GTK_EXTEXT_STYLE_UNDER)
          text[count++] = 'u';
       text[count] = '\0';
-      if(HighlightCache.flags[i] & STYLE_STRIKE)
+      if(HighlightCache.flags[i] & GTK_EXTEXT_STYLE_STRIKE)
          text[count++] = 's';
       text[count] = '\0';
       g_snprintf(string, sizeof(string), "/" PACKAGE "/Highlighting/%s.class", editor_color_names[i]);
@@ -445,10 +516,10 @@ void save_default_colors(void)
       if(DefaultCache.types[i] == FONT_BOLD || DefaultCache.types[i] == FONT_BOLD_ITALIC)
          text[count++] = 'b';
       text[count] = '\0';
-      if(DefaultCache.flags[i] & STYLE_UNDER)
+      if(DefaultCache.flags[i] & GTK_EXTEXT_STYLE_UNDER)
          text[count++] = 'u';
       text[count] = '\0';
-      if(DefaultCache.flags[i] & STYLE_STRIKE)
+      if(DefaultCache.flags[i] & GTK_EXTEXT_STYLE_STRIKE)
          text[count++] = 's';
       text[count] = '\0';
       g_snprintf(string, sizeof(string), "/" PACKAGE "/Highlighting/%s.class", editor_color_names[i]);
@@ -565,10 +636,10 @@ void build_default_colors(void)
    DefaultCache.types[CHAR_STRING] = FONT_NORMAL;
    DefaultCache.flags[CHAR_STRING] = 0;
 
-   DefaultCache.colors[CHAR] = DefaultCache.colors[DARK_BLUE];
-   DefaultCache.fonts[CHAR] = get_font(FONT_NORMAL);
-   DefaultCache.types[CHAR] = FONT_NORMAL;
-   DefaultCache.flags[CHAR] = 0;
+   DefaultCache.colors[S_CHAR] = DefaultCache.colors[DARK_BLUE];
+   DefaultCache.fonts[S_CHAR] = get_font(FONT_NORMAL);
+   DefaultCache.types[S_CHAR] = FONT_NORMAL;
+   DefaultCache.flags[S_CHAR] = 0;
 
    DefaultCache.colors[COMMENT] = DefaultCache.colors[DARK_GREEN];
    DefaultCache.fonts[COMMENT] = get_font(FONT_ITALIC);
