@@ -1,4 +1,4 @@
-/*  getprefs.h
+/*  Glimmer - getprefs.h
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -14,13 +14,15 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ifndef _GETPREFS_H
-#define _GETPREFS_H
+#ifndef _GETPREFS_H_
+#define _GETPREFS_H_
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+#include "declarations.h"
+
 void get_prefs(void);
 void save_prefs(void);
 void save_defaults(void);
