@@ -1,4 +1,4 @@
-/*  highlightbar.h
+/*  Glimmer - highlightbar.h
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -14,19 +14,22 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#ifndef HIGHLIGHTBAR_H
-#define HIGHLIGHTBAR_H
+#ifndef _HIGHLIGHTBAR_H_
+#define _HIGHLIGHTBAR_H_
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+#include "declarations.h"
+#include "main.h"
+
 void make_highlight_bar(GtkWidget *menu);
 
 extern GtkWidget *highlightbar;
-extern GtkWidget *highlightbar_option;
 extern GtkWidget *rehighlight_button;
 extern GtkWidget *reparse_button;
+extern GtkWidget *highlightbar_option;
 
 #ifdef __cplusplus
 }
