@@ -1,4 +1,4 @@
-/*  linebar.h
+/*  Glimmer - linebar.h
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -21,6 +21,8 @@
 extern "C" {
 #endif
 
+#include "declarations.h"
+
 void make_line_toolbar(void);
 void execute_line_cb(GtkWidget* widget, gpointer data);
 void line_combo_set_history(GList *history);
