@@ -1,4 +1,4 @@
-/*  macro-language.h
+/*  Glimmer - macro-language.h
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -21,20 +21,23 @@
 extern "C" {
 #endif
 
-extern void initialize_macro_language(int argc, char *argv[]);
-extern void execute_macro_string(char* string);
+#include "declarations.h"
+#include "bindings.h"
+
+void initialize_macro_language(int argc, char *argv[]);
+
+#ifdef WITH_PYTHON
+void execute_macro_string(char* string);
 void autoexec_script(void);
 void build_scripts_menu(void);
 void script_run_cb(GtkWidget *widget, gchar *filename);
 void regenerate_scripts_cb(GtkWidget *widget, gpointer data);
-
 void adjust_popup_menu(void);
 void adjust_build_menu(void);
-
 void remove_pyref(GtkWidget *widget, PyObject *object);
 void call_python_function_cb(GtkWidget *widget, gpointer *function);
-
 void init_scripting(void);
+#endif
 
 extern GList *dynamic_build;
 extern GList *edit_entries;
