@@ -1,4 +1,4 @@
-/*  mdi-routines.h
+/*  Glimmer - mdi-routines.h
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -21,9 +21,13 @@
 extern "C" {
 #endif
 
+#include "declarations.h"
+#include "widgets/gdsfile.h"
+
 GdsFile *create_file(gchar *name);
 void file_pos_changed(GtkWidget *widget, gint pos, gpointer data);
 void gds_file_focus_with_raise(GtkWidget *widget, GdsFile *gds_file);
+void set_length_depends(gint length);
 
 #ifdef __cplusplus
 }
