@@ -1,7 +1,29 @@
+/*  Glimmer - searches.c
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Library General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
+
 #include "declarations.h"
+#include "searches.h"
+#include "dialogs.h"
 #include "findbar.h"
 #include "linebar.h"
+#include "main.h"
+#include "misc.h"
+#include "settings.h"
 #include "windows.h"
+#include "widgets/gdsfile.h"
 
 static void add_manpage_string(GtkWidget *widget, gpointer data);
 
@@ -16,18 +38,16 @@ static void toggle_button_switch(GtkWidget *widget, gint *value);
 void match_bracket_cb(GtkWidget *widget, gpointer data)
 {
    GtkExText *text;
-   GtkEditable *editable;
    gchar cur_char;
    gint start = 0;
 
    g_return_if_fail(cur_file != NULL);
    text = GTK_EXTEXT(cur_file->text);
-   editable = GTK_EDITABLE(cur_file->text);
-   start = editable->current_pos - 1;
+   start = text->current_pos - 1;
    cur_char = GTK_EXTEXT_INDEX(text, start);
    if(!strchr("<{[()]}>", cur_char)) return;
    if(find_bracket_match(cur_file->text, &start))
-      gtk_extext_set_position(editable, start+1);
+      gtk_extext_set_position(text, start+1);
 }
 
 /* 
@@ -78,7 +98,7 @@ void find_text_cb(GtkWidget *widget, GnomeSearchDialog *search_dialog)
    gchar text[256];
    g_return_if_fail(cur_file != NULL);
 
-   old_pos = gtk_extext_get_position(GTK_EDITABLE(cur_file->text));
+   old_pos = gtk_extext_get_position(GTK_EXTEXT(cur_file->text));
    search_string = gnome_search_dialog_get_find_text(GNOME_SEARCH_DIALOG(search_dialog));
    search_string = g_strdup(search_string);
    if(search_start == GNOME_SEARCH_START_TOP) pos = 0;
@@ -111,7 +131,7 @@ void find_text_cb(GtkWidget *widget, GnomeSearchDialog *search_dialog)
          index++;
          file = get_nth_file_proper(index);
          if(file) file_change(NULL, file);
-         gtk_extext_set_position(GTK_EDITABLE(cur_file->text), 0);
+         gtk_extext_set_position(GTK_EXTEXT(cur_file->text), 0);
          while(gtk_events_pending()) gtk_main_iteration();
          gtk_signal_emit_by_name(GTK_OBJECT(search_dialog->find_button), "clicked", NULL);
       }
@@ -132,17 +152,17 @@ void find_text_cb(GtkWidget *widget, GnomeSearchDialog *search_dialog)
 /*
  * Find some text in a GtkExText with the given settings
  */
-gboolean find_text(GtkExText *extext, gint start, gint end, gchar *search_string, gboolean case_sensitive, gboolean is_regex, gboolean select, GtkExTextMatch *match)
+gboolean find_text(GtkExText *text, gint start, gint end, gchar *search_string, gboolean case_sensitive, gboolean is_regex, gboolean select, GtkExTextMatch *match)
 {
    Regex regex;
    gint pos = 0;
    gboolean ret;
-   g_return_val_if_fail(extext != NULL, FALSE);
+   g_return_val_if_fail(text != NULL, FALSE);
    if(is_regex)
    {
       if(gds_editor_compile_regex(search_string, &regex))
       {
-         pos = gtk_extext_regex_search(extext, start, &regex, TRUE, match);
+         pos = gtk_extext_regex_search(text, start, &regex, TRUE, match);
          if(pos == -1) pos = 0;
          if(regex.buf.fastmap) g_free(regex.buf.fastmap);
          regex.buf.fastmap = NULL;
@@ -151,16 +171,16 @@ gboolean find_text(GtkExText *extext, gint start, gint end, gchar *search_string
    }
    else
    {
-      pos = gtk_extext_search(extext, search_string, start, case_sensitive, TRUE, match);
+      pos = gtk_extext_search(text, search_string, start, case_sensitive, TRUE, match);
    }
    ret = pos && match->endpos <= end ? TRUE : FALSE;
    if(ret && select)
    {
-      gtk_extext_set_position(GTK_EDITABLE(extext), match->endpos);
-      gtk_editable_select_region(GTK_EDITABLE(extext), match->startpos, match->endpos);
+      gtk_extext_set_position(GTK_EXTEXT(text), match->endpos);
+      gtk_extext_select_region(GTK_EXTEXT(text), match->startpos, match->endpos);
    }
    else if(ret)
-      gtk_extext_set_position(GTK_EDITABLE(extext), match->endpos);
+      gtk_extext_set_position(GTK_EXTEXT(text), match->endpos);
    return(ret);
 }
 
@@ -232,9 +252,9 @@ void replace_all_text_cb(GtkWidget *widget, GnomeSearchDialog *search_dialog)
    search_text = gnome_search_dialog_get_find_text(search_dialog);
    new_text = gnome_search_dialog_get_replace_text(search_dialog);
    if(search_dialog->search_selection)
-      gtk_editable_set_position(GTK_EDITABLE(cur_file->text), search_dialog->startpos);
+      gtk_extext_set_position(GTK_EXTEXT(cur_file->text), search_dialog->startpos);
    else if(search_start == GNOME_SEARCH_START_TOP)
-      gtk_editable_set_position(GTK_EDITABLE(cur_file->text), 0);
+      gtk_extext_set_position(GTK_EXTEXT(cur_file->text), 0);
    gtk_extext_freeze(GTK_EXTEXT(cur_file->text));
    replacements = replace_all_text(GTK_EXTEXT(cur_file->text), search_text, new_text, case_sensitive, search_style, search_dialog->search_selection ? search_dialog->endpos : GTK_EXTEXT(cur_file->text)->length);
    gtk_extext_thaw(GTK_EXTEXT(cur_file->text));
@@ -255,22 +275,20 @@ void replace_all_text_cb(GtkWidget *widget, GnomeSearchDialog *search_dialog)
    }
 }
 
-gint replace_all_text(GtkExText *extext, gchar *search_text, gchar *text, gboolean iscase, gboolean regex, gint end)
+gint replace_all_text(GtkExText *text, gchar *search_text, gchar *new_text, gboolean iscase, gboolean regex, gint end)
 {
-   GtkEditable *editable;
    GtkExTextMatch match;
-   gint replacements = 0;   
+   gint replacements = 0;
 
-   editable = GTK_EDITABLE(extext);
    while(1)
    {
-      if(find_text(extext, editable->current_pos, end, search_text, iscase, regex, TRUE, &match))
+      if(find_text(text, text->current_pos, end, search_text, iscase, regex, TRUE, &match))
       {
-         replace_text(GTK_EXTEXT(cur_file->text), text);
-         end += (strlen(text) - strlen(search_text));
+         replace_text(GTK_EXTEXT(cur_file->text), new_text);
+         end += (strlen(new_text) - strlen(search_text));
          replacements++;
       }
-      else if(search_multi)
+      else if(search_multi && end != GTK_EXTEXT(cur_file->text)->length)
       {
          GdsFile *file;
          gint total;
@@ -280,16 +298,15 @@ gint replace_all_text(GtkExText *extext, gchar *search_text, gchar *text, gboole
          index = get_nth_file_number(cur_file);
          if(index != total-1)
          {
-            gtk_extext_thaw(extext);
+            gtk_extext_thaw(text);
             index++;
             file = get_nth_file_proper(index);
             if(file) file_change(NULL, file);
-            gtk_extext_set_position(GTK_EDITABLE(file->text), 0);
+            gtk_extext_set_position(GTK_EXTEXT(file->text), 0);
             while(gtk_events_pending()) gtk_main_iteration();
-            extext = GTK_EXTEXT(cur_file->text);
-            editable = GTK_EDITABLE(extext);
-            end = gtk_extext_get_length(extext);
-            gtk_extext_freeze(extext);
+            text = GTK_EXTEXT(cur_file->text);
+            end = gtk_extext_get_length(text);
+            gtk_extext_freeze(text);
          }
          else break;
       }
@@ -301,22 +318,20 @@ gint replace_all_text(GtkExText *extext, gchar *search_text, gchar *text, gboole
 /*
  * Replace the current selection in editable with replace_text
  */
-gboolean replace_text(GtkExText *extext, gchar *text)
+gboolean replace_text(GtkExText *text, gchar *new_text)
 {
    gint start, end;
-   GtkEditable *editable;
-   editable = GTK_EDITABLE(extext);
-   if(editable->has_selection)
+   if(text->has_selection)
    {
-      start = editable->selection_start_pos;
-      end = editable->selection_end_pos;
-      editable->has_selection = FALSE;
-      (gint)editable->selection_start_pos = -1;
-      (gint)editable->selection_end_pos = -1;
-      gtk_editable_delete_text(editable, start, end);
-      gtk_editable_set_position(editable, start);
-      if(text && strlen(text)>0)
-         gtk_editable_insert_text(editable, text, strlen(text), (gint *)&editable->current_pos);
+      start = text->selection_start_pos;
+      end = text->selection_end_pos;
+      text->has_selection = FALSE;
+      text->selection_start_pos = -1;
+      text->selection_end_pos = -1;
+      gtk_extext_delete_text(text, start, end);
+      gtk_extext_set_position(text, start);
+      if(new_text && strlen(new_text)>0)
+         gtk_extext_insert_text(text, new_text, strlen(new_text), (gint *)&text->current_pos);
       return(TRUE);
    }
    return(FALSE);
@@ -327,21 +342,21 @@ gboolean replace_text(GtkExText *extext, gchar *text)
  */
 void gotoline_cb(GtkWidget *widget, gpointer data)
 {
-   gchar *text=NULL;
+   gchar *line_text=NULL;
    gint line_num;
-   GtkExText *extext;
+   GtkExText *text;
    g_return_if_fail(cur_file != NULL);
    
-   extext = GTK_EXTEXT(cur_file->text);
-   text = gtk_entry_get_text(GTK_ENTRY(line_entry));
-   if(strlen(text)>0)
+   text = GTK_EXTEXT(cur_file->text);
+   line_text = gtk_entry_get_text(GTK_ENTRY(line_entry));
+   if(strlen(line_text)>0)
    {
-      if(sscanf(text, "%d", &line_num) && line_num >= 1 && line_num <= extext->line_count)
+      if(sscanf(line_text, "%d", &line_num) && line_num >= 1 && line_num <= text->line_count)
       {
-         add_string_to_file("line.hist", text);
-         line_combo_add_string(text);
-         gtk_extext_set_line(extext, line_num-1);
-         gtk_widget_grab_focus(GTK_WIDGET(extext));
+         add_string_to_file("line.hist", line_text);
+         line_combo_add_string(line_text);
+         gtk_extext_set_line(text, line_num-1);
+         gtk_widget_grab_focus(GTK_WIDGET(text));
       }
       else display_message("Line not found.", FLASH);
    }
@@ -388,22 +403,22 @@ static void add_manpage_string(GtkWidget *widget, gpointer data)
 
 void manpage_cb(GtkWidget *widget, gpointer data)
 {
-   GtkEditable *editable;
+   GtkExText *extext;
    gchar *text = NULL;
    gint start, end;
 
-   editable = GTK_EDITABLE(cur_file->text);
+   extext = GTK_EXTEXT(cur_file->text);
    
-   if(editable->has_selection)
+   if(extext->has_selection)
    {
-      text = gtk_editable_get_chars(editable, editable->selection_start_pos, editable->selection_end_pos);
+      text = gtk_extext_get_chars(extext, extext->selection_start_pos, extext->selection_end_pos);
    }
    else
    {
-      start = gtk_editable_get_position(GTK_EDITABLE(cur_file->text));
-      if(gtk_extext_get_current_word(GTK_EXTEXT(cur_file->text), &start, &end))
+      start = gtk_extext_get_position(extext);
+      if(gtk_extext_get_current_word(extext, &start, &end))
       {
-         text = gtk_editable_get_chars(editable, start, end);
+         text = gtk_extext_get_chars(extext, start, end);
       }
    }
    if(GPOINTER_TO_INT(data) == 0)
@@ -447,20 +462,20 @@ void set_find_text_from_selection(GnomeSearchDialog *sd)
    guint sel_end;
    gchar *sel_text;
 
-   if(GTK_EDITABLE(cur_file->text)->has_selection)
+   if(GTK_EXTEXT(cur_file->text)->has_selection)
    {
-      sel_start = GTK_EDITABLE(cur_file->text)->selection_start_pos;
-      sel_end = GTK_EDITABLE(cur_file->text)->selection_end_pos;
+      sel_start = GTK_EXTEXT(cur_file->text)->selection_start_pos;
+      sel_end = GTK_EXTEXT(cur_file->text)->selection_end_pos;
       if(sel_end-sel_start <= 48)
       {
-         sel_text = gtk_editable_get_chars(GTK_EDITABLE(cur_file->text), sel_start, sel_end);
+         sel_text = gtk_extext_get_chars(GTK_EXTEXT(cur_file->text), sel_start, sel_end);
          gnome_search_dialog_set_find_text(sd, sel_text);         
          gnome_search_dialog_search_selection(sd, FALSE);
          g_free(sel_text);
       }	
       else
       {
-         GTK_EDITABLE(cur_file->text)->current_pos = sel_start;
+         GTK_EXTEXT(cur_file->text)->current_pos = sel_start;
          gnome_search_dialog_set_startpos(sd, sel_start);
          gnome_search_dialog_set_endpos(sd, sel_end);
          gnome_search_dialog_search_selection(sd, TRUE);
@@ -476,6 +491,7 @@ void last_bookmark_cb(GtkWidget *widget, gpointer data)
 {
    GtkExText *text;
    GdkPixmap *pixmap;
+   GdkBitmap *mask;
    gint line;
    gboolean found = FALSE;
 
@@ -484,8 +500,8 @@ void last_bookmark_cb(GtkWidget *widget, gpointer data)
    while(line > 0)
    {
       line--;
-      pixmap = gtk_extext_get_line_pixmap(text, line);
-      if(pixmap == bookmark_pixmap)
+      gtk_extext_get_line_pixmap(text, line, &pixmap, &mask);
+      if(pixmap == bookmark_pixmap->pixmap)
       {
          found = TRUE;
          break;
@@ -502,6 +518,7 @@ void next_bookmark_cb(GtkWidget *widget, gpointer data)
 {
    GtkExText *text;
    GdkPixmap *pixmap;
+   GdkBitmap *mask;
    gint line;
    gboolean found = FALSE;
 
@@ -510,8 +527,8 @@ void next_bookmark_cb(GtkWidget *widget, gpointer data)
    while(line < text->line_count)
    {
       line++;
-      pixmap = gtk_extext_get_line_pixmap(text, line);
-      if(pixmap == bookmark_pixmap)
+      gtk_extext_get_line_pixmap(text, line, &pixmap, &mask);
+      if(pixmap == bookmark_pixmap->pixmap)
       {
          found = TRUE;
          break;
