@@ -1,4 +1,4 @@
-/*  session-management.h
+/*  Glimmer - session-management.h
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -21,6 +21,8 @@
 extern "C" {
 #endif
 
+#include "declarations.h"
+
 void init_session_management(void);
 void session_die(GnomeClient *client, gpointer data);
 
