@@ -1,4 +1,4 @@
-/*  session.c
+/*  Glimmer - session.c
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
@@ -21,6 +21,9 @@
 extern "C" {
 #endif
 
+#include "declarations.h"
+#include "widgets/gnomefilelist.h"
+
 guint get_last_session(void);
 void save_session_cb(GtkWidget *widget, gpointer data);
 void open_session_cb(GtkWidget *widget, gpointer data);
