@@ -1,10 +1,34 @@
-#include <gdk/gdkprivate.h>
-#include <gtk/gtkprivate.h>
+/*  Glimmer - settings.c
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Library General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
+
+#include <libgnomeprint/gnome-font.h>
+#include <libgnomeprint/gnome-font-dialog.h>
+
 #include "declarations.h"
+#include "settings.h"
+#include "file-io.h"
+#include "getprefs.h"
+#include "main.h"
+#include "menus.h"
+#include "misc.h"
+#include "syntax-highlight.h"
+
 #include "widgets/gnomedirentry.h"
 #include "widgets/gnome-settings.h"
 
-#include "xpm/glimmer.xpm"
 #include "xpm/folder_closed.xpm"
 #include "xpm/window.xpm"
 
@@ -23,13 +47,17 @@ static void apply_settings_changes(void);
 void apply_window_settings_cb(GtkWidget *widget, gpointer data);
 void get_window_settings_cb(GtkWidget *widget, gpointer data);
 static void select_ctree_row(GtkCTree *tree, gint row, gint col, GdkEvent *event, gpointer data);
+static void gnome_font_set_preview(GnomeFontSelection *font_sel, GnomeFont *font);
+static void widget_sensitivity_callback(GtkWidget *toggle, GtkWidget *target);
 
+
+// Global preferences data
 Preferences general_preferences;
 Preferences backup_preferences;
 Preferences default_preferences;
 
-// The main window
-static GtkWidget *settings_window;
+// The main settings window
+static GtkWidget *settings_window = NULL;
 
 // General Settings
 static gint settings_selected_row = 2;
@@ -79,10 +107,12 @@ static GtkWidget *fg = NULL;
 static gboolean fg_changed = FALSE;
 static GtkWidget *bg = NULL;
 static gboolean bg_changed = FALSE;
-static GtkWidget *bm_bg = NULL;
-static gboolean bm_bg_changed = FALSE;
 static GtkWidget *s_bg = NULL;
 static gboolean s_bg_changed = FALSE;
+static GtkWidget *bm_bg = NULL;
+static gboolean bm_bg_changed = FALSE;
+static GtkWidget *gutter_bg = NULL;
+static gboolean gutter_bg_changed = FALSE;
 static GtkWidget *bg_pixmap = NULL;
 static gboolean bg_pixmap_changed = FALSE;
 
@@ -126,6 +156,20 @@ static GtkWidget *win_y_spin;
 static GtkWidget *win_w_spin;
 static GtkWidget *win_h_spin;
 
+//Print Settings
+static GtkWidget *print_portrait;
+static GtkWidget *print_landscape;
+static GtkWidget *print_header;
+static GtkWidget *print_numbers;
+static GtkWidget *print_n_lines;
+static GtkWidget *print_with_wrap;
+static GtkWidget *print_font;
+static GtkWidget *print_font_preview;
+
+static gboolean iprint_header;
+static gboolean iprint_numbers;
+static gboolean iprint_with_wrap;
+
 void edit_preferences(GtkWidget *widget, gpointer data)
 {
    GtkWidget *main_box;
@@ -154,9 +198,12 @@ void edit_preferences(GtkWidget *widget, gpointer data)
    GtkWidget *tabs_box;
    GtkWidget *window_frame;
    GtkWidget *window_box;
+   GtkWidget *print_frame;
+   GtkWidget *print_box;
   
    GtkWidget *util_box;
    GtkWidget *util_label;
+   GtkWidget *util_frame;
 
    GtkWidget *buttons_box;
    GtkWidget *ok_button;
@@ -185,10 +232,12 @@ void edit_preferences(GtkWidget *widget, gpointer data)
    font_changed = FALSE;
    italic_changed = FALSE;
    bold_changed = FALSE;
+   boldi_changed = FALSE;
    fg_changed = FALSE;
    bg_changed = FALSE;
-   bm_bg_changed = FALSE;
    s_bg_changed = FALSE;
+   bm_bg_changed = FALSE;
+   gutter_bg_changed = FALSE;
    bg_pixmap_changed = FALSE;
 
    isave_toolbars = general_preferences.save_toolbars;
@@ -209,6 +258,9 @@ void edit_preferences(GtkWidget *widget, gpointer data)
    isyntax = general_preferences.syntax;
    iauto_indent = general_preferences.auto_indent;
    iuse_spaces = general_preferences.use_spaces;
+   iprint_header = general_preferences.print_header;
+   iprint_numbers = general_preferences.print_numbers;
+   iprint_with_wrap = general_preferences.print_with_wrap;
 
    settings_window = gtk_window_new(GTK_WINDOW_DIALOG);
    gtk_container_set_border_width(GTK_CONTAINER(settings_window), 5);   
@@ -222,8 +274,9 @@ void edit_preferences(GtkWidget *widget, gpointer data)
    gtk_box_pack_start(GTK_BOX(main_box), settings, TRUE, TRUE, 0);
    gtk_widget_show(settings);
 
-   pixmap = (GnomePixmap *)gnome_pixmap_new_from_xpm_d_at_size(glimmer_xpm, 18, 18);
-   topnode = gnome_settings_add_page(GNOME_SETTINGS(settings), NULL, PACKAGE, pixmap, NULL);
+   //pixmap = (GnomePixmap *)gnome_pixmap_new_from_xpm_d_at_size(glimmer_xpm, 18, 18);
+   //topnode = gnome_settings_add_page(GNOME_SETTINGS(settings), NULL, PACKAGE, pixmap, NULL);
+   topnode = NULL;
    pixmap = (GnomePixmap *)gnome_stock_pixmap_widget_at_size(GTK_WIDGET(settings_window), GNOME_STOCK_PIXMAP_BOOK_OPEN, 18, 18);
    pnode = gnome_settings_add_page(GNOME_SETTINGS(settings), topnode, "General", pixmap, NULL);
 
@@ -302,7 +355,6 @@ void edit_preferences(GtkWidget *widget, gpointer data)
    gtk_box_pack_start(GTK_BOX(util_box), undo_spin, TRUE, TRUE, 0);   
    gtk_widget_show(undo_spin);
 
-
    util_box = gtk_hbox_new(FALSE, 0);
    gtk_container_set_border_width(GTK_CONTAINER(util_box), 5);
    gtk_widget_show(util_box);
@@ -439,7 +491,7 @@ void edit_preferences(GtkWidget *widget, gpointer data)
    gtk_box_pack_start(GTK_BOX(util_box), util_label, FALSE, FALSE, 0);         
    gtk_widget_show(util_label);
    font = gnome_font_picker_new();
-   gtk_signal_connect(GTK_OBJECT(font), "font_set", GTK_SIGNAL_FUNC(set_font_changed), 0);
+   gtk_signal_connect(GTK_OBJECT(font), "font_set", GTK_SIGNAL_FUNC(set_font_changed), &font_changed);
    gnome_font_picker_set_mode(GNOME_FONT_PICKER(font), GNOME_FONT_PICKER_MODE_FONT_INFO);
    gnome_font_picker_set_preview_text(GNOME_FONT_PICKER(font), PACKAGE " v" VERSION);
    gnome_font_picker_fi_set_use_font_in_label(GNOME_FONT_PICKER(font), TRUE, 14);
@@ -456,7 +508,7 @@ void edit_preferences(GtkWidget *widget, gpointer data)
    gtk_box_pack_start(GTK_BOX(util_box), util_label, FALSE, FALSE, 0);         
    gtk_widget_show(util_label);
    italic = gnome_font_picker_new();
-   gtk_signal_connect(GTK_OBJECT(italic), "font_set", GTK_SIGNAL_FUNC(set_font_changed), 0);
+   gtk_signal_connect(GTK_OBJECT(italic), "font_set", GTK_SIGNAL_FUNC(set_font_changed), &italic_changed);
    gnome_font_picker_set_mode(GNOME_FONT_PICKER(italic), GNOME_FONT_PICKER_MODE_FONT_INFO);
    gnome_font_picker_set_preview_text(GNOME_FONT_PICKER(italic), PACKAGE " v" VERSION);
    gnome_font_picker_fi_set_use_font_in_label(GNOME_FONT_PICKER(italic), TRUE, 14);
@@ -473,7 +525,7 @@ void edit_preferences(GtkWidget *widget, gpointer data)
    gtk_box_pack_start(GTK_BOX(util_box), util_label, FALSE, FALSE, 0);         
    gtk_widget_show(util_label);
    bold = gnome_font_picker_new();
-   gtk_signal_connect(GTK_OBJECT(bold), "font_set", GTK_SIGNAL_FUNC(set_font_changed), 0);
+   gtk_signal_connect(GTK_OBJECT(bold), "font_set", GTK_SIGNAL_FUNC(set_font_changed), &bold_changed);
    gnome_font_picker_set_mode(GNOME_FONT_PICKER(bold), GNOME_FONT_PICKER_MODE_FONT_INFO);
    gnome_font_picker_set_preview_text(GNOME_FONT_PICKER(bold), PACKAGE " v" VERSION);
    gnome_font_picker_fi_set_use_font_in_label(GNOME_FONT_PICKER(bold), TRUE, 14);
@@ -490,7 +542,7 @@ void edit_preferences(GtkWidget *widget, gpointer data)
    gtk_box_pack_start(GTK_BOX(util_box), util_label, FALSE, FALSE, 0);         
    gtk_widget_show(util_label);
    boldi = gnome_font_picker_new();
-   gtk_signal_connect(GTK_OBJECT(boldi), "font_set", GTK_SIGNAL_FUNC(set_font_changed), 0);
+   gtk_signal_connect(GTK_OBJECT(boldi), "font_set", GTK_SIGNAL_FUNC(set_font_changed), &boldi_changed);
    gnome_font_picker_set_mode(GNOME_FONT_PICKER(boldi), GNOME_FONT_PICKER_MODE_FONT_INFO);
    gnome_font_picker_set_preview_text(GNOME_FONT_PICKER(boldi), PACKAGE " v" VERSION);
    gnome_font_picker_fi_set_use_font_in_label(GNOME_FONT_PICKER(boldi), TRUE, 14);
@@ -545,7 +597,7 @@ void edit_preferences(GtkWidget *widget, gpointer data)
    gtk_box_pack_start(GTK_BOX(util_box), util_label, FALSE, FALSE, 0);         
    gtk_widget_show(util_label);
    fg = gnome_color_picker_new();
-   gtk_signal_connect(GTK_OBJECT(fg), "color_set", GTK_SIGNAL_FUNC(set_color_changed), 0);
+   gtk_signal_connect(GTK_OBJECT(fg), "color_set", GTK_SIGNAL_FUNC(set_color_changed), &fg_changed);
    gnome_color_picker_set_dither(GNOME_COLOR_PICKER(fg), FALSE);   
    gnome_color_picker_set_use_alpha(GNOME_COLOR_PICKER(fg), FALSE);   
    gtk_box_pack_end(GTK_BOX(util_box), fg, FALSE, FALSE, 0);
@@ -558,7 +610,7 @@ void edit_preferences(GtkWidget *widget, gpointer data)
    gtk_box_pack_start(GTK_BOX(util_box), util_label, FALSE, FALSE, 0);         
    gtk_widget_show(util_label);
    bg = gnome_color_picker_new();
-   gtk_signal_connect(GTK_OBJECT(bg), "color_set", GTK_SIGNAL_FUNC(set_color_changed), 0);
+   gtk_signal_connect(GTK_OBJECT(bg), "color_set", GTK_SIGNAL_FUNC(set_color_changed), &bg_changed);
    gnome_color_picker_set_dither(GNOME_COLOR_PICKER(bg), FALSE);   
    gnome_color_picker_set_use_alpha(GNOME_COLOR_PICKER(bg), FALSE);   
    gtk_box_pack_end(GTK_BOX(util_box), bg, FALSE, FALSE, 0);
@@ -571,7 +623,7 @@ void edit_preferences(GtkWidget *widget, gpointer data)
    gtk_box_pack_start(GTK_BOX(util_box), util_label, FALSE, FALSE, 0);         
    gtk_widget_show(util_label);
    s_bg = gnome_color_picker_new();
-   gtk_signal_connect(GTK_OBJECT(s_bg), "color_set", GTK_SIGNAL_FUNC(set_color_changed), 0);
+   gtk_signal_connect(GTK_OBJECT(s_bg), "color_set", GTK_SIGNAL_FUNC(set_color_changed), &s_bg_changed);
    gnome_color_picker_set_dither(GNOME_COLOR_PICKER(s_bg), FALSE);   
    gnome_color_picker_set_use_alpha(GNOME_COLOR_PICKER(s_bg), FALSE);   
    gtk_box_pack_end(GTK_BOX(util_box), s_bg, FALSE, FALSE, 0);
@@ -584,12 +636,25 @@ void edit_preferences(GtkWidget *widget, gpointer data)
    gtk_box_pack_start(GTK_BOX(util_box), util_label, FALSE, FALSE, 0);         
    gtk_widget_show(util_label);
    bm_bg = gnome_color_picker_new();
-   gtk_signal_connect(GTK_OBJECT(bm_bg), "color_set", GTK_SIGNAL_FUNC(set_color_changed), 0);
+   gtk_signal_connect(GTK_OBJECT(bm_bg), "color_set", GTK_SIGNAL_FUNC(set_color_changed), &bm_bg_changed);
    gnome_color_picker_set_dither(GNOME_COLOR_PICKER(bm_bg), FALSE);   
    gnome_color_picker_set_use_alpha(GNOME_COLOR_PICKER(bm_bg), FALSE);   
    gtk_box_pack_end(GTK_BOX(util_box), bm_bg, FALSE, FALSE, 0);
    gtk_widget_show(bm_bg);
 
+   util_box  = gtk_hbox_new(FALSE, 0);
+   gtk_box_pack_start(GTK_BOX(color_box), util_box, FALSE, TRUE, 0);         
+   gtk_widget_show(util_box);
+   util_label = gtk_label_new(_("Gutter Background Color:"));
+   gtk_box_pack_start(GTK_BOX(util_box), util_label, FALSE, FALSE, 0);         
+   gtk_widget_show(util_label);
+   gutter_bg = gnome_color_picker_new();
+   gtk_signal_connect(GTK_OBJECT(gutter_bg), "color_set", GTK_SIGNAL_FUNC(set_color_changed), &gutter_bg_changed);
+   gnome_color_picker_set_dither(GNOME_COLOR_PICKER(gutter_bg), FALSE);   
+   gnome_color_picker_set_use_alpha(GNOME_COLOR_PICKER(gutter_bg), FALSE);   
+   gtk_box_pack_end(GTK_BOX(util_box), gutter_bg, FALSE, FALSE, 0);
+   gtk_widget_show(gutter_bg);
+
    bg_pixmap = gnome_pixmap_entry_new(PACKAGE"-Background Image", "Change Image", 1);
    gtk_box_pack_start(GTK_BOX(color_box), bg_pixmap, FALSE, FALSE, 10);
    gnome_pixmap_entry_set_preview_size(GNOME_PIXMAP_ENTRY(bg_pixmap), 100, 100);
@@ -752,8 +817,65 @@ void edit_preferences(GtkWidget *widget, gpointer data)
    gtk_signal_connect(GTK_OBJECT(get_window_settings), "clicked", GTK_SIGNAL_FUNC(get_window_settings_cb), 0);
    gtk_widget_show(get_window_settings);
 
-   /* Buttons Box */
+   /* Printing Settings */
+   util_box = gtk_hbox_new(FALSE, 0);
+   gtk_container_set_border_width(GTK_CONTAINER(util_box), 5);
+   gtk_widget_show(util_box);
+   pixmap = (GnomePixmap *)gnome_stock_pixmap_widget_at_size(GTK_WIDGET(settings_window), GNOME_STOCK_PIXMAP_PRINT, 18, 18);
+   pnode = gnome_settings_add_page(GNOME_SETTINGS(settings), topnode, _("Print Settings"), pixmap, util_box);
 
+   print_frame = gtk_frame_new(_("Print Settings"));
+   gtk_box_pack_start(GTK_BOX(util_box), print_frame, TRUE, TRUE, 0);
+   gtk_widget_show(print_frame);
+   print_box = gtk_vbox_new(FALSE, 0);
+   gtk_container_set_border_width(GTK_CONTAINER(print_box), 5);
+   gtk_container_add(GTK_CONTAINER(print_frame), print_box);
+   gtk_widget_show(print_box);
+
+   print_font = gnome_font_selection_new();
+   gtk_box_pack_start(GTK_BOX(print_box), print_font, FALSE, FALSE, 0);
+   gtk_widget_show(print_font);
+
+   print_font_preview = gnome_font_preview_new();
+   gtk_box_pack_start(GTK_BOX(print_box), print_font_preview, FALSE, FALSE, 0);
+   gtk_signal_connect(GTK_OBJECT(print_font), "font_set", GTK_SIGNAL_FUNC(gnome_font_set_preview), NULL);
+   gtk_widget_show(print_font_preview);
+
+   util_frame = gtk_frame_new(_("Paper Orientation"));
+   gtk_container_set_border_width(GTK_CONTAINER(print_frame), 5);
+   gtk_box_pack_start(GTK_BOX(print_box), util_frame, FALSE, FALSE, 0);
+   gtk_widget_show(util_frame);
+   util_box = gtk_hbox_new(FALSE, 0);
+   gtk_container_add(GTK_CONTAINER(util_frame), util_box);
+   gtk_widget_show(util_box);
+
+   print_portrait = gtk_radio_button_new_with_label(NULL, "Portrait");
+   gtk_box_pack_start(GTK_BOX(util_box), print_portrait, FALSE, FALSE, 0);
+   gtk_widget_show(print_portrait);
+   print_landscape = gtk_radio_button_new_with_label(GTK_RADIO_BUTTON(print_portrait)->group, "Landscape");
+   gtk_box_pack_start(GTK_BOX(util_box), print_landscape, FALSE, FALSE, 0);
+   gtk_widget_show(print_landscape);
+
+   print_header = make_button(_("Print headers?"), print_box, &iprint_header);
+   print_with_wrap = make_button(_("Print with word wrap?"), print_box, &iprint_with_wrap);
+   print_numbers = make_button(_("Print line numbers?"), print_box, &iprint_numbers);
+
+   util_box = gtk_hbox_new(FALSE, 5);
+   gtk_box_pack_start(GTK_BOX(print_box), util_box, FALSE, FALSE, 0);
+   gtk_widget_show(util_box);
+   util_label = gtk_label_new(_("Print numbers every:"));
+   gtk_box_pack_start(GTK_BOX(util_box), util_label, FALSE, FALSE, 0);   
+   gtk_widget_show(util_label);
+   print_n_lines =  gtk_spin_button_new(GTK_ADJUSTMENT(gtk_adjustment_new(0, 1, 25, 1, 5, 0)), 1, 0);
+   gtk_box_pack_start(GTK_BOX(util_box), print_n_lines, TRUE, TRUE, 0);   
+   gtk_widget_show(print_n_lines);
+   util_label = gtk_label_new(_("lines."));
+   gtk_box_pack_start(GTK_BOX(util_box), util_label, FALSE, FALSE, 0);   
+   gtk_widget_show(util_label);
+   gtk_widget_set_sensitive(print_n_lines, iprint_numbers);
+   gtk_signal_connect(GTK_OBJECT(print_numbers), "toggled", GTK_SIGNAL_FUNC(widget_sensitivity_callback), util_box);
+
+   /* Buttons Box */
    hsep = gtk_hseparator_new();
    gtk_box_pack_start(GTK_BOX(main_box), hsep, FALSE, TRUE, 10);
    gtk_widget_show(hsep);
@@ -819,14 +941,15 @@ static void apply_settings(GtkWidget *widget, gpointer data)
 static void get_defaults(GtkWidget *widget, gpointer data)
 {
    put_preferences(&default_preferences);
-   gtk_signal_emit_by_name(GTK_OBJECT(fg), "color_set", 0);
-   gtk_signal_emit_by_name(GTK_OBJECT(bg), "color_set", 0);
-   gtk_signal_emit_by_name(GTK_OBJECT(bm_bg), "color_set", 0);
-   gtk_signal_emit_by_name(GTK_OBJECT(s_bg), "color_set", 0);
-   gtk_signal_emit_by_name(GTK_OBJECT(font), "font_set", 0);
-   gtk_signal_emit_by_name(GTK_OBJECT(italic), "font_set", 0);
-   gtk_signal_emit_by_name(GTK_OBJECT(bold), "font_set", 0);
-   gtk_signal_emit_by_name(GTK_OBJECT(boldi), "font_set", 0);
+   gtk_signal_emit_by_name(GTK_OBJECT(font), "font_set", &font_changed);
+   gtk_signal_emit_by_name(GTK_OBJECT(italic), "font_set", &italic_changed);
+   gtk_signal_emit_by_name(GTK_OBJECT(bold), "font_set", &bold_changed);
+   gtk_signal_emit_by_name(GTK_OBJECT(boldi), "font_set", &boldi_changed);
+   gtk_signal_emit_by_name(GTK_OBJECT(fg), "color_set", &fg_changed);
+   gtk_signal_emit_by_name(GTK_OBJECT(bg), "color_set", &bg_changed);
+   gtk_signal_emit_by_name(GTK_OBJECT(s_bg), "color_set", &s_bg_changed);
+   gtk_signal_emit_by_name(GTK_OBJECT(bm_bg), "color_set", &bm_bg_changed);
+   gtk_signal_emit_by_name(GTK_OBJECT(gutter_bg), "color_set", &gutter_bg_changed);
    if(default_preferences.use_gtk_theme != general_preferences.use_gtk_theme)
       gtk_signal_emit_by_name(GTK_OBJECT(gtk_theme), "clicked", &iuse_gtk_theme);
 }
@@ -843,27 +966,30 @@ static void get_theme_defaults(GtkWidget *widget, gpointer data)
    style = text_widget->style;
    gnome_color_picker_set_i16(GNOME_COLOR_PICKER(fg), style->text[GTK_STATE_NORMAL].red, style->text[GTK_STATE_NORMAL].green, style->text[GTK_STATE_NORMAL].blue, 0);
    gnome_color_picker_set_i16(GNOME_COLOR_PICKER(bg), style->bg[GTK_STATE_NORMAL].red, style->bg[GTK_STATE_NORMAL].green, style->bg[GTK_STATE_NORMAL].blue, 0);
-   gnome_color_picker_set_i16(GNOME_COLOR_PICKER(bm_bg), style->bg[GTK_STATE_PRELIGHT].red, style->bg[GTK_STATE_PRELIGHT].green, style->bg[GTK_STATE_PRELIGHT].blue, 0);
    gnome_color_picker_set_i16(GNOME_COLOR_PICKER(s_bg), style->bg[GTK_STATE_SELECTED].red, style->bg[GTK_STATE_SELECTED].green, style->bg[GTK_STATE_SELECTED].blue, 0);
-   gtk_signal_emit_by_name(GTK_OBJECT(fg), "color_set", 0);
-   gtk_signal_emit_by_name(GTK_OBJECT(bg), "color_set", 0);
-   gtk_signal_emit_by_name(GTK_OBJECT(bm_bg), "color_set", 0);
-   gtk_signal_emit_by_name(GTK_OBJECT(s_bg), "color_set", 0);
-   gtk_signal_emit_by_name(GTK_OBJECT(font), "font_set", 0);
+   gnome_color_picker_set_i16(GNOME_COLOR_PICKER(bm_bg), style->bg[GTK_STATE_PRELIGHT].red, style->bg[GTK_STATE_PRELIGHT].green, style->bg[GTK_STATE_PRELIGHT].blue, 0);
+   gnome_color_picker_set_i16(GNOME_COLOR_PICKER(gutter_bg), style->bg[GTK_STATE_INSENSITIVE].red, style->bg[GTK_STATE_INSENSITIVE].green, style->bg[GTK_STATE_INSENSITIVE].blue, 0);
+   gtk_signal_emit_by_name(GTK_OBJECT(font), "font_set", &font_changed);
+   gtk_signal_emit_by_name(GTK_OBJECT(fg), "color_set", &fg_changed);
+   gtk_signal_emit_by_name(GTK_OBJECT(bg), "color_set", &bg_changed);
+   gtk_signal_emit_by_name(GTK_OBJECT(s_bg), "color_set", &s_bg_changed);
+   gtk_signal_emit_by_name(GTK_OBJECT(bm_bg), "color_set", &bm_bg_changed);
+   gtk_signal_emit_by_name(GTK_OBJECT(gutter_bg), "color_set", &gutter_bg_changed);
    gtk_widget_destroy(win);
 }
 
 static void get_backups(GtkWidget *widget, gpointer data)
 {
    put_preferences(&backup_preferences);
-   gtk_signal_emit_by_name(GTK_OBJECT(fg), "color_set", 0);
-   gtk_signal_emit_by_name(GTK_OBJECT(bg), "color_set", 0);
-   gtk_signal_emit_by_name(GTK_OBJECT(bm_bg), "color_set", 0);
-   gtk_signal_emit_by_name(GTK_OBJECT(s_bg), "color_set", 0);
-   gtk_signal_emit_by_name(GTK_OBJECT(font), "font_set", 0);
-   gtk_signal_emit_by_name(GTK_OBJECT(italic), "font_set", 0);
-   gtk_signal_emit_by_name(GTK_OBJECT(bold), "font_set", 0);
-   gtk_signal_emit_by_name(GTK_OBJECT(boldi), "font_set", 0);
+   gtk_signal_emit_by_name(GTK_OBJECT(font), "font_set", &font_changed);
+   gtk_signal_emit_by_name(GTK_OBJECT(italic), "font_set", &italic_changed);
+   gtk_signal_emit_by_name(GTK_OBJECT(bold), "font_set", &bold_changed);
+   gtk_signal_emit_by_name(GTK_OBJECT(boldi), "font_set", &boldi_changed);
+   gtk_signal_emit_by_name(GTK_OBJECT(fg), "color_set", &fg_changed);
+   gtk_signal_emit_by_name(GTK_OBJECT(bg), "color_set", &bg_changed);
+   gtk_signal_emit_by_name(GTK_OBJECT(s_bg), "color_set", &s_bg_changed);
+   gtk_signal_emit_by_name(GTK_OBJECT(bm_bg), "color_set", &bm_bg_changed);
+   gtk_signal_emit_by_name(GTK_OBJECT(gutter_bg), "color_set", &gutter_bg_changed);
    if(backup_preferences.use_gtk_theme != iuse_gtk_theme)
       gtk_signal_emit_by_name(GTK_OBJECT(gtk_theme), "clicked", &iuse_gtk_theme);
 }
@@ -871,7 +997,9 @@ static void get_backups(GtkWidget *widget, gpointer data)
 static void get_preferences(Preferences *source)
 {
    GdkFont *old_font;
+   GnomeFont *gnome_font;
    gchar *text;
+
    text = gnome_direntry_get_dir(GNOME_DIRENTRY(default_dir_entry));
    strncpy(source->default_dir, text, sizeof(source->default_dir));
    text = gnome_direntry_get_dir(GNOME_DIRENTRY(projects_dir_entry));
@@ -974,8 +1102,9 @@ static void get_preferences(Preferences *source)
    source->use_gtk_theme = iuse_gtk_theme;
    gnome_color_picker_get_i16(GNOME_COLOR_PICKER(fg), &source->fg.red, &source->fg.green, &source->fg.blue, NULL);
    gnome_color_picker_get_i16(GNOME_COLOR_PICKER(bg), &source->bg.red, &source->bg.green, &source->bg.blue, NULL);
-   gnome_color_picker_get_i16(GNOME_COLOR_PICKER(bm_bg), &source->bm_bg.red, &source->bm_bg.green, &source->bm_bg.blue, NULL);
    gnome_color_picker_get_i16(GNOME_COLOR_PICKER(s_bg), &source->s_bg.red, &source->s_bg.green, &source->s_bg.blue, NULL);
+   gnome_color_picker_get_i16(GNOME_COLOR_PICKER(bm_bg), &source->bm_bg.red, &source->bm_bg.green, &source->bm_bg.blue, NULL);
+   gnome_color_picker_get_i16(GNOME_COLOR_PICKER(gutter_bg), &source->gutter_bg.red, &source->gutter_bg.green, &source->gutter_bg.blue, NULL);
 
    text = gnome_pixmap_entry_get_filename(GNOME_PIXMAP_ENTRY(bg_pixmap));
    if(text) strncpy(general_preferences.pixmap_name, text, 256);
@@ -1001,11 +1130,28 @@ static void get_preferences(Preferences *source)
    source->y = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(win_y_spin));
    source->w = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(win_w_spin));
    source->h = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(win_h_spin));
+
+   gnome_font = gnome_font_selection_get_font (GNOME_FONT_SELECTION(print_font));
+   gnome_font_ref(gnome_font);
+   text = gnome_font_get_full_name(GNOME_FONT(gnome_font));
+   if(text && strlen(text))
+   {
+      strcpy(source->print_font, text);
+      g_free(text);
+   }
+   gnome_font_unref(gnome_font);
+   source->print_landscape = get_which_radio_active(GTK_RADIO_BUTTON(print_landscape)->group);;
+   source->print_header = iprint_header;
+   source->print_with_wrap = iprint_with_wrap;
+   source->print_numbers = iprint_numbers;
+   source->print_every_n_lines = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(print_n_lines));
 }
 
 static void put_preferences(Preferences *source)
 {
    GtkWidget *entry;
+   GnomeFont *gnome_font;
+
    gnome_direntry_set_dir(GNOME_DIRENTRY(default_dir_entry), source->default_dir);
    gnome_direntry_set_dir(GNOME_DIRENTRY(projects_dir_entry), source->projects_dir);
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(save_toolbars), source->save_toolbars);
@@ -1045,8 +1191,9 @@ static void put_preferences(Preferences *source)
       boldi_changed = TRUE;
    gnome_color_picker_set_i16(GNOME_COLOR_PICKER(fg), source->fg.red, source->fg.green, source->fg.blue, 0);
    gnome_color_picker_set_i16(GNOME_COLOR_PICKER(bg), source->bg.red, source->bg.green, source->bg.blue, 0);
-   gnome_color_picker_set_i16(GNOME_COLOR_PICKER(bm_bg), source->bm_bg.red, source->bm_bg.green, source->bm_bg.blue, 0);
    gnome_color_picker_set_i16(GNOME_COLOR_PICKER(s_bg), source->s_bg.red, source->s_bg.green, source->s_bg.blue, 0);
+   gnome_color_picker_set_i16(GNOME_COLOR_PICKER(bm_bg), source->bm_bg.red, source->bm_bg.green, source->bm_bg.blue, 0);
+   gnome_color_picker_set_i16(GNOME_COLOR_PICKER(gutter_bg), source->gutter_bg.red, source->gutter_bg.green, source->gutter_bg.blue, 0);
 
    entry = gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(GNOME_PIXMAP_ENTRY(bg_pixmap)->fentry));
    gtk_entry_set_text(GTK_ENTRY(entry), source->pixmap_name);
@@ -1061,10 +1208,6 @@ static void put_preferences(Preferences *source)
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(fullselection), source->full_selection);
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(bracketmatch), source->bracketmatch);
    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(select_line), source->select_line);
-   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(syntax), source->syntax);
-   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(auto_indent), source->auto_indent);
-   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(tab_stops), source->tab_stops);
-   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(use_spaces), source->use_spaces);
    ifullfilenamestitle = source->full_filenames_title;
    ifullfilenames = source->full_filenames;
    icloseall = source->close_all;
@@ -1075,6 +1218,11 @@ static void put_preferences(Preferences *source)
    ifullselection = source->full_selection;
    ibracketmatch = source->bracketmatch;
    iselect_line = source->select_line;
+
+   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(syntax), source->syntax);
+   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(auto_indent), source->auto_indent);
+   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(tab_stops), source->tab_stops);
+   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(use_spaces), source->use_spaces);
    isyntax = source->syntax;
    iauto_indent = source->auto_indent;
    iuse_spaces = source->use_spaces;
@@ -1086,6 +1234,19 @@ static void put_preferences(Preferences *source)
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(win_y_spin), (gfloat)source->y);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(win_w_spin), (gfloat)source->w);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(win_h_spin), (gfloat)source->h);
+
+   gnome_font = gnome_font_new_from_full_name((const gchar *)source->print_font);
+   gnome_font_ref(gnome_font);
+   gnome_font_selection_set_font(GNOME_FONT_SELECTION(print_font), gnome_font);
+   gnome_font_unref(gnome_font);
+   set_which_radio_active(GTK_RADIO_BUTTON(print_landscape)->group, source->print_landscape);
+   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(print_header), source->print_header);
+   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(print_with_wrap), source->print_with_wrap);
+   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(print_numbers), source->print_numbers);
+   gtk_spin_button_set_value(GTK_SPIN_BUTTON(print_n_lines), (gfloat)source->print_every_n_lines);
+   iprint_header = source->print_header;
+   iprint_with_wrap = source->print_with_wrap;
+   iprint_numbers = source->print_numbers;
 }
 
 static void copy_preferences(Preferences *source, Preferences *dest)
@@ -1095,18 +1256,29 @@ static void copy_preferences(Preferences *source, Preferences *dest)
 
 static void set_font_changed(GtkWidget *gfp, gchar *font_name, gpointer data)
 {
+   gboolean *value;
+   value = data;
+   *value = TRUE;
+/*
    if(gfp == font) font_changed = TRUE;
    else if(gfp == italic) italic_changed = TRUE;
    else if(gfp == bold) bold_changed = TRUE;
    else if(gfp == boldi) boldi_changed = TRUE;
+*/
 }
 
 static void set_color_changed(GtkWidget *cp, guint r, guint g, guint b, guint a, gpointer data)
 {
+   gboolean *value;
+   value = data;
+   *value = TRUE;
+/*
    if(cp == bg) bg_changed = TRUE;
    else if(cp == fg) fg_changed = TRUE;
    else if(cp == s_bg) s_bg_changed = TRUE;
    else if(cp == bm_bg) bm_bg_changed = TRUE;
+   else if(cp == bm_bg) bm_bg_changed = TRUE;
+*/
 }
 
 static void toggle_button_switch(GtkWidget *widget, gint *value)
@@ -1122,11 +1294,12 @@ static void gtk_theme_switch_cb(GtkWidget *widget, gint *value)
       gtk_widget_set_sensitive(color_frame, TRUE);
    else
       gtk_widget_set_sensitive(color_frame, FALSE);
+   font_changed = TRUE;
    bg_changed = TRUE;
    fg_changed = TRUE;
    s_bg_changed = TRUE;
    bm_bg_changed = TRUE;
-   font_changed = TRUE;
+   gutter_bg_changed = TRUE;
 }
 
 static void apply_settings_changes(void)
@@ -1160,6 +1333,7 @@ static void apply_settings_changes(void)
       extext_style->bg[GTK_STATE_NORMAL] = general_preferences.bg;
       extext_style->bg[GTK_STATE_SELECTED] = general_preferences.s_bg;
       extext_style->bg[GTK_STATE_PRELIGHT] = general_preferences.bm_bg;
+      extext_style->bg[GTK_STATE_INSENSITIVE] = general_preferences.gutter_bg;
       extext_style->bg_pixmap[GTK_STATE_NORMAL] = NULL;
       if(strlen(general_preferences.pixmap_name))
       {
@@ -1258,7 +1432,7 @@ static void apply_settings_changes(void)
          gtk_extext_set_tab_width(GTK_EXTEXT(temp->text), general_preferences.spaces);
       }
       gtk_extext_set_select_line(GTK_EXTEXT(temp->text), general_preferences.select_line);
-      gtk_extext_set_position(GTK_EDITABLE(temp->text), GTK_EDITABLE(temp->text)->current_pos);
+      gtk_extext_set_position(GTK_EXTEXT(temp->text), GTK_EXTEXT(temp->text)->current_pos);
       file = file->next;
    }
    setup_recent_files_list(general_preferences.history);
@@ -1313,3 +1487,18 @@ static void select_ctree_row(GtkCTree *tree, gint row, gint col, GdkEvent *event
 {
    settings_selected_row = row;
 }
+
+static void gnome_font_set_preview(GnomeFontSelection *font_sel, GnomeFont *font)
+{
+   const gchar *str;
+   str = gnome_font_get_name(font);
+   gnome_font_preview_set_font(GNOME_FONT_PREVIEW(print_font_preview), font);
+   gnome_font_preview_set_phrase(GNOME_FONT_PREVIEW(print_font_preview), str);
+}
+
+static void widget_sensitivity_callback(GtkWidget *toggle, GtkWidget *target)
+{
+   gboolean active;
+   active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toggle));
+   gtk_widget_set_sensitive(target, active);
+}
