@@ -1,10 +1,34 @@
+/*  Glimmer - syntax-highlight.c
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Library General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
+
+#include <libgnomevfs/gnome-vfs-mime.h>
+
 #include "declarations.h"
+#include "syntax-highlight.h"
+#include "colors.h"
 #include "insert.h"
 #include "macro-language.h"
-#include "colors.h"
+#include "main.h"
+#include "misc.h"
+#include "settings.h"
 #include "signals.h"
 
-GSList *highlight_options = NULL;
+void g_strconvescape(gchar *source);
+
+GList *highlight_options = NULL;
 GList *highlight_tables = NULL;
 GdsFileHighlightTable *parse_table = NULL;
 gboolean parsing_language = FALSE;
@@ -26,15 +50,15 @@ GtkWidget *build_highlight_menu(void)
    GtkWidget *menuitem = NULL;
    gchar *text;
    gchar buffer[1024];
-   gchar lang_string[] = "language=";
-   gchar *filename;
+   gchar *filename = NULL;
    gchar home_dir[384] = "";
    gchar install_dir[] = LANG_DIR;
 
    g_snprintf(home_dir, sizeof(home_dir), "%s/." PACKAGE "/languages", getenv("HOME"));   
    menu = gtk_menu_new();
-   none_item = gtk_radio_menu_item_new_with_label(NULL, "None");
+   none_item = gtk_menu_item_new_with_label("None");
    gtk_object_set_data(GTK_OBJECT(none_item), "filename", "none"); 
+   highlight_options = g_list_append(highlight_options, (gpointer)none_item);
    gtk_signal_connect(GTK_OBJECT(none_item), "activate", GTK_SIGNAL_FUNC(highlight_table_change_cb), GINT_TO_POINTER(1));
    gtk_menu_append(GTK_MENU(menu), none_item);
    gtk_widget_show(none_item);
@@ -45,7 +69,8 @@ GtkWidget *build_highlight_menu(void)
    for(cur = g_list_first(filenames); cur; cur = cur->next)
    {
       filename = (gchar *)cur->data;
-      if(!(file = fopen(filename, "r"))) continue;
+      if(!(file = fopen(filename, "r")))
+         continue;
       while(fgets(buffer, sizeof(buffer), file))
       {
          if(buffer[0]=='#') continue;
@@ -55,22 +80,26 @@ GtkWidget *build_highlight_menu(void)
             {
                if(strstr(buffer, "</settings>")) break;
                if(strstr(buffer, "\n")) buffer[strlen(buffer)-1] = '\0';
-               if((text = strstr(buffer, lang_string)))
+               if((text = strstr(buffer, "<language>")))
                {
-                  text = g_strdup(text + strlen(lang_string));
+                  gchar *end;
+                  text = text + strlen("<language>");
+                  end = strstr(text, "</language>");
+                  if(end) *end = '\0';
+                  text = g_strdup(text);
                   if(!check_list_for_string(names, text))
                   {
-                     menuitem = gtk_radio_menu_item_new_with_label(GTK_RADIO_MENU_ITEM(none_item)->group, text);
-                     gtk_object_set_data(GTK_OBJECT(menuitem), "filename", (gpointer)filename); 
+                     menuitem = gtk_menu_item_new_with_label(text);
+                     highlight_options = g_list_append(highlight_options, (gpointer)menuitem);
+                     gtk_object_set_data(GTK_OBJECT(menuitem), "filename", (gpointer)g_strdup(filename));
                      gtk_signal_connect(GTK_OBJECT(menuitem), "activate", GTK_SIGNAL_FUNC(highlight_table_change_cb), GINT_TO_POINTER(1));
                      gtk_menu_append(GTK_MENU(menu), menuitem);
                      gtk_widget_show(menuitem);
                      names = g_list_append(names, (gpointer)text);
+                     continue;
                   }
                   else
                   {
-                     g_free(filename);
-                     cur->data = NULL;
                      g_free(text);
                   }
                }
@@ -79,13 +108,8 @@ GtkWidget *build_highlight_menu(void)
       }
       fclose(file);
    }
-   g_list_free(filenames);
-   for(cur = g_list_first(names); cur; cur = cur->next)
-   {
-      g_free(cur->data);
-   }
-   g_list_free(names);
-   highlight_options = GTK_RADIO_MENU_ITEM(none_item)->group;
+   g_list_delete_all(filenames);
+   g_list_delete_all(names);
    return(menu);
 }
 
@@ -107,16 +131,18 @@ GList *build_file_listing(gchar *directory, GList *filenames)
    gchar *full_path = NULL;
 
    dir_list = opendir(directory);
-   if(dir_list != NULL)
+   if(dir_list)
    {
       while((dir = readdir(dir_list)) != NULL)
       {
          if(dir->d_ino > 0)
          {
+            /* Still a possible mem leak starting here. */
             full_path = g_strconcat(directory, "/", dir->d_name, NULL);
             filenames = g_list_append(filenames, (gpointer)full_path);
          }
       }
+      closedir(dir_list);
    }
    return(filenames);
 }
@@ -181,6 +207,7 @@ void attach_highlight_tables(GdsFile *file_ptr, gchar *path_to_file)
    gchar *extension = NULL;
    gchar *filename = NULL;   
    gchar install_dir[] = LANG_DIR;
+   gchar *mime_type;
    GList *temp = NULL;
    GdsFileHighlightTable *current;
 
@@ -198,7 +225,16 @@ void attach_highlight_tables(GdsFile *file_ptr, gchar *path_to_file)
             break;
          }
       }
-      if(!table) table = make_highlight_tables(path_to_file);
+      if(!table)
+      {
+         xmlDocPtr doc;   
+         doc = parse_glimmer_definition(path_to_file);
+         if(doc)
+         {
+            table = make_highlight_tables(doc, path_to_file);
+            xmlFreeDoc(doc);
+         }
+      }
    }
    else
    {
@@ -206,8 +242,10 @@ void attach_highlight_tables(GdsFile *file_ptr, gchar *path_to_file)
       extension = strrchr(filename, '.');
       if(extension) extension++;
       else extension = filename;
-      table = find_highlight_tables(extension, home_dir);  // try the local path first
-      if(!table) table = find_highlight_tables(extension, install_dir);
+      mime_type = (gchar *) gnome_vfs_mime_type_from_name(filename);
+      g_print("Extension: %s, mime-type: %s.\n", extension, mime_type);
+      table = find_highlight_tables(extension, mime_type, home_dir);
+      if(!table) table = find_highlight_tables(extension, mime_type, install_dir);
    }
 
    if(table)
@@ -218,8 +256,10 @@ void attach_highlight_tables(GdsFile *file_ptr, gchar *path_to_file)
       if(table->pattern) gds_editor_install_table(GDS_EDITOR(file_ptr->text), table->pattern);
       if(table->embedded) gds_editor_install_table(GDS_EDITOR(file_ptr->text), table->embedded);
       gds_editor_set_highlight(GDS_EDITOR(file_ptr->text), general_preferences.syntax);
+#ifdef WITH_PYTHON
       if(table->table_add_callback) table_emit_scripting_signal("table-add", file_ptr->tables);
       if(table->file_focus_callback) file_emit_scripting_signal("focus-in", file_ptr);
+#endif
    }
    else
    {
@@ -232,11 +272,13 @@ gboolean detach_highlight_tables(GdsFile *file_ptr)
 {
    if(file_ptr->tables != NULL)
    {
+#ifdef WITH_PYTHON
       if(file_ptr->tables->file_unfocus_callback) file_emit_scripting_signal("focus-out", file_ptr);
       if(file_ptr->tables->table_remove_callback) table_emit_scripting_signal("table-remove", file_ptr->tables);
       if(file_ptr->compile_pid == -1 && file_ptr->tables->stop_compile_hook) file_emit_scripting_signal("stop-compile", file_ptr);
       if(file_ptr->debug_pid == -1 && file_ptr->tables->stop_debug_hook) file_emit_scripting_signal("stop-debug", file_ptr);
       if(file_ptr->exec_pid == -1 && file_ptr->tables->stop_execute_hook) file_emit_scripting_signal("stop-execute", file_ptr);
+#endif
       file_ptr->tables->refcount--;
       delete_highlight_tables(file_ptr->tables);
       file_ptr->tables = NULL;
@@ -251,20 +293,25 @@ gboolean detach_highlight_tables(GdsFile *file_ptr)
 
 gboolean delete_highlight_tables(GdsFileHighlightTable *table)
 {
+#ifdef WITH_PYTHON
    GList *list = NULL;
+#endif
    if(table->refcount > 0) return(FALSE);
    if(table->syntax) gds_editor_table_free(table->syntax);
    if(table->pattern) gds_editor_table_free(table->pattern);
    if(table->embedded) gds_editor_table_free(table->embedded);
    g_free(table->filename);
    g_list_delete_all(table->extensions);
+   g_list_delete_all(table->mime_types);
    g_free(table->props.compiler);
    g_free(table->props.debugger);
    g_free(table->props.execution);
-   if(table->table_destroy_callback)
-      table_emit_scripting_signal("table-destroy", table);
+#ifdef WITH_PYTHON
+   if(table->table_destroy_callback) table_emit_scripting_signal("table-destroy", table);
    remove_signal_hooks(table);
+#endif
    highlight_tables = g_list_remove(highlight_tables, table);
+#ifdef WITH_PYTHON
    table->menu_entries = g_list_reverse(table->menu_entries);
    for(list = g_list_first(table->menu_entries); list; list = list->next)
    {
@@ -290,11 +337,12 @@ gboolean delete_highlight_tables(GdsFileHighlightTable *table)
       }
    }
    g_list_free(table->toolbar_entries);
+#endif
    g_free(table);
    return(TRUE);
 }
 
-GdsFileHighlightTable *find_highlight_tables(gchar *extension, gchar *cur_dir)
+GdsFileHighlightTable *find_highlight_tables(gchar *extension, gchar *mime_type, gchar *cur_dir)
 {
    GList *temp = NULL;
    GList *ptr = NULL;
@@ -303,10 +351,9 @@ GdsFileHighlightTable *find_highlight_tables(gchar *extension, gchar *cur_dir)
    struct dirent *dir;
    gchar buffer[256];
    gchar full_path[384];
-   GList *exts = NULL;
-   gboolean ext = FALSE;
-   GdsFileHighlightTable *new_table;   
-   
+   GdsFileHighlightTable *new_table = NULL;
+   xmlDocPtr doc;   
+
    for(temp = g_list_first(highlight_tables); temp; temp = temp->next)
    {
       current = (GdsFileHighlightTable *)temp->data;
@@ -317,10 +364,17 @@ GdsFileHighlightTable *find_highlight_tables(gchar *extension, gchar *cur_dir)
             return(current);
          }
       }
+      for(ptr = g_list_first(current->mime_types); ptr; ptr = ptr->next)
+      {
+         if(!strcmp((gchar *)ptr->data, mime_type))
+         {
+            return(current);
+         }
+      }
    }
 
    dir_list = opendir(cur_dir);
-   if(dir_list != NULL)
+   if(dir_list)
    {
       while((dir = readdir(dir_list)) != NULL)
       {
@@ -338,102 +392,165 @@ GdsFileHighlightTable *find_highlight_tables(gchar *extension, gchar *cur_dir)
             }
             if(found) continue;
             g_snprintf(full_path, sizeof(full_path), "%s/%s", cur_dir, dir->d_name);
-            if(!(exts = get_exts_list(full_path)))
+            doc = parse_glimmer_definition(full_path);
+            if(doc)
             {
-               continue;
-            }
-            ext = check_exts_list(extension, exts);
-            g_list_delete_all(exts);
-            if(ext)
-            {
-               new_table = make_highlight_tables(full_path);
-               return(new_table);
-            }
-         }
-      }
-   }
-   return(NULL);
-}
-
-
-GList *get_exts_list(gchar *filename)
-{
-   FILE *file;
-   gchar buffer[256];
-   const char end_char = '\"';
-   gchar name[32];
-   gchar ext[24];   
-   GList *temp = NULL;
-   gchar *new_ext;
-   gint success = 0;
-   
-   if(!(file = fopen(filename, "r")))
-      return(NULL);
-
-   while(fgets(buffer, sizeof(buffer), file) != NULL)
-   {
-      if(strstr(buffer, "<extensions>") != NULL) 
-      {
-         while(fgets(buffer, sizeof(buffer), file) != NULL)
-         {
-            if(strstr(buffer, "</extensions>")) break;
-            if((success = get_str(buffer, name, "name=\"", end_char, sizeof(name))))
-            {
-               if((success = get_str(buffer, ext, "ext=\"", end_char, sizeof(ext))))
+               if(check_mime_type(doc, extension, mime_type))
                {
-                  new_ext = g_new(char, strlen(ext)+1);
-                  strcpy(new_ext, ext);
-                  temp = g_list_append(temp, (gpointer)new_ext);
+                  new_table = make_highlight_tables(doc, full_path);
+                	xmlFreeDoc(doc);
+                	break;
+               }
+               else
+               {
+                	xmlFreeDoc(doc);
                }
             }
          }
       }
+      closedir(dir_list);
    }
-   fclose(file);
-   return(temp);
+   return(new_table);
 }
 
-gboolean check_exts_list(gchar *ext, GList *exts)
+xmlDocPtr parse_glimmer_definition(gchar *filename)
 {
-   GList *temp = NULL;
-   gchar *current;
-   temp = g_list_first(exts);
-   while(temp)
-   {
-      current = (gchar *)temp->data;
-      if(!strcmp(ext, current))
-         return(TRUE);
-      temp = temp->next;
-   }
-   return(FALSE);
-}
-
-GdsFileHighlightTable *make_highlight_tables(gchar *filename)
-{
-   FILE *ifile;
-   FILE *ofile;
-   gchar temp_file[384];
-   GList *list = NULL;
-   GdsFileHighlightTable *new_table;
+   xmlDocPtr doc = NULL;
+   FILE *infile;
+   FILE *outfile;
    gchar buffer[2048];
-   gchar name[32];
-   gchar start[1024];
-   gchar end[1024];
-   gchar words[1536];
-   gchar color[32];
-   gchar cclass[8];
-   gchar default_class[] = "d";
-   const char end_char = '\"';
+   gchar *tempfile;
+   gint line_count = 0;
 
-   gchar *text;
-   gchar compiler_string[] = "compiler=";
-   gchar debugger_string[] = "debugger=";
-   gchar execution_string[] = "execution=";
-   gchar ext[24];   
-   gchar *new_ext;
-   gint success = FALSE;
+   if(strstr(filename, "languages/."))
+      return(NULL);
 
-   if(!(ifile = fopen(filename, "r"))) return(NULL);
+   tempfile = g_strconcat(getenv("HOME"), "/." PACKAGE "/temp.xml", NULL);
+
+   if(!(infile = fopen(filename, "r")))
+      return(NULL);
+
+   if(!(outfile = fopen(tempfile, "w")))
+   {
+      g_free(tempfile);
+      fclose(infile);
+      return(NULL);
+   }
+   while(fgets(buffer, sizeof(buffer), infile) != NULL)
+   {
+      if(strstr(buffer, "<glimmer-definition>") != NULL) 
+      {
+         do
+         {
+            fprintf(outfile, "%s", buffer);
+            line_count++;
+            if(strstr(buffer, "</glimmer-definition>")) break;
+         } while(fgets(buffer, sizeof(buffer), infile) != NULL);
+      }
+   }
+   fclose(infile);
+   fclose(outfile);
+   if(line_count)
+   {
+    	doc = xmlParseFile(tempfile);
+   }
+ 	g_free(tempfile);
+   return(doc);
+}
+
+gboolean check_mime_type(xmlDocPtr doc, gchar *extension, gchar *mime_type)
+{
+	xmlNodePtr root;
+   xmlNodePtr node;
+   char *namestring;
+   gboolean found = FALSE;
+
+	root = xmlDocGetRootElement(doc);
+   node = root->xmlChildrenNode;
+   if(!node)
+   {
+      return(found);
+   }
+   if(strcmp(node->name, "settings"))
+   {
+      return(found);
+   }
+   node = node->next;
+   if(strcmp(node->name, "mime-types"))
+   {
+      return(found);
+   }
+   for(node = node->xmlChildrenNode; node != NULL; node = node->next)
+   {
+      if(!strcmp(node->name, "extension"))
+      {
+         namestring = xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
+         if(!strcmp(namestring, extension))
+         {
+            found = TRUE;
+         }
+         g_free(namestring);
+      }
+      else if(!strcmp(node->name, "mime-type"))
+      {
+         namestring = xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
+         if(!strcmp(namestring, mime_type))
+         {
+            found = TRUE;
+         }
+         g_free(namestring);
+      }
+      if(found) break;
+   }
+   return(found);
+}
+
+void g_strconvescape(gchar *source)
+{
+   char cur_char;
+   char last_char = '\0';
+   gint iterations = 0;
+   gint max_chars;
+   gchar *dest;
+
+   max_chars = strlen(source);
+   dest = source;
+
+   for(iterations = 0; iterations < max_chars; iterations++)
+   {
+      cur_char = source[iterations];
+      *dest = cur_char;
+      if(last_char == '\\' && cur_char == 'n')
+      {
+         dest--;
+         *dest = '\n';
+      }
+      else if(last_char == '\\' && cur_char == 't')
+      {
+         dest--;
+         *dest = '\t';
+      }
+      last_char = cur_char;
+      dest++;
+   }
+   *dest = '\0';
+}
+
+GdsFileHighlightTable *make_highlight_tables(xmlDocPtr doc, gchar *filename)
+{
+   GdsFileHighlightTable *new_table;
+	xmlNodePtr root;
+   xmlNodePtr section;
+   xmlNodePtr node;
+   xmlNodePtr item;
+   gchar *name = NULL;
+   gchar *start = NULL;
+   gchar *end = NULL;
+   gchar *regex = NULL;
+   gchar *style = NULL;
+   gchar *cclass = NULL;
+   gchar *text = NULL;
+   gchar *default_class = "d";
 
    new_table = (GdsFileHighlightTable *)g_malloc0(sizeof(GdsFileHighlightTable));
    parse_table = new_table;
@@ -449,194 +566,253 @@ GdsFileHighlightTable *make_highlight_tables(gchar *filename)
    new_table->props.debugger = NULL;
    new_table->props.execution = NULL;
 
-   while(fgets(buffer, sizeof(buffer), ifile))
+   root = xmlDocGetRootElement(doc);
+   for(section = root->xmlChildrenNode; section != NULL; section = section->next)
    {
-      if(buffer[0] == '#') continue;
-      if(strstr(buffer, "<settings>")) 
+      if(!strcmp(section->name, "settings"))
       {
-         while(fgets(buffer, sizeof(buffer), ifile))
+         for(node = section->xmlChildrenNode; node != NULL; node = node->next)
          {
-            if(strstr(buffer, "</settings>")) break;
-            if(strstr(buffer, "\n")) buffer[strlen(buffer)-1] = '\0';
-            if((text = strstr(buffer, compiler_string)))
+            if(!strcmp(node->name, "language"))
             {
-               new_table->props.compiler = g_strdup(text + strlen(compiler_string));
-               success++;
-               continue;
+               
             }
-            else if((text = strstr(buffer, debugger_string)))
+            else if(!strcmp(node->name, "compiler"))
             {
-               new_table->props.debugger = g_strdup(text + strlen(debugger_string));
-               success++;
-               continue;
+               new_table->props.compiler = xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
             }
-            else if((text = strstr(buffer, execution_string)))
+            else if(!strcmp(node->name, "debugger"))
             {
-               new_table->props.execution = g_strdup(text + strlen(execution_string));
-               success++;
-               continue;
+               new_table->props.debugger = xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
             }
-            else if(sscanf(buffer, "over_ride=%d", &new_table->props.over_ride))
+            else if(!strcmp(node->name, "execution"))
             {
-               success++;
-               continue;
+               new_table->props.execution = xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
             }
-            else if(sscanf(buffer, "auto_indent=%d", &new_table->props.auto_indent))
+            else if(!strcmp(node->name, "auto-indent"))
             {
-               success++;
-               continue;
-            }
-            else if(sscanf(buffer, "use_spaces=%d", &new_table->props.use_spaces))
-            {
-               success++;
-               continue;
-            }
-            else if(sscanf(buffer, "tab_stop=%d", &new_table->props.spaces))
-            {
-               success++;
-            }
-         }
-      }
-      else if(strstr(buffer, "<extensions>"))
-      {
-         while(fgets(buffer, sizeof(buffer), ifile))
-         {
-            if(strstr(buffer, "</extensions>")) break;
-            if((success = get_str(buffer, name, "name=\"", end_char, sizeof(name))))
-            {
-               if((success = get_str(buffer, ext, "ext=\"", end_char, sizeof(ext))))
+               text = xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
+               if(text)
                {
-                  new_ext = g_strdup(ext);
-                  new_table->extensions = g_list_append(new_table->extensions, (gpointer)new_ext);
+                  new_table->props.auto_indent = atoi(text);
+                  g_free(text);
+               }
+            }
+            else if(!strcmp(node->name, "use-spaces"))
+            {
+               text = xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
+               if(text)
+               {
+                  new_table->props.use_spaces = atoi(text);
+                  g_free(text);
+               }
+            }
+            else if(!strcmp(node->name, "tab-stop"))
+            {
+               text = xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
+               if(text)
+               {
+                  new_table->props.spaces = atoi(text);
+                  g_free(text);
+               }
+            }
+            else if(!strcmp(node->name, "over-ride"))
+            {
+               text = xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
+               if(text)
+               {
+                  new_table->props.over_ride = atoi(text);
+                  g_free(text);
                }
             }
          }
       }
-      else if(strstr(buffer, "<syntax>"))
+      else if(!strcmp(section->name, "mime-types"))
       {
-         list = NULL;
-         while(fgets(buffer, sizeof(buffer), ifile))
+         for(node = section->xmlChildrenNode; node != NULL; node = node->next)
          {
-            if(buffer[0] == '#') continue;
-            if(strstr(buffer, "</syntax>")) break;
-            else
+            if(!strcmp(node->name, "extension"))
             {
-               if((success = get_str(buffer, name, "name=\"", end_char, sizeof(name))))
+               text = xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
+               if(text)
                {
-                  if((success = get_str(buffer, start, "start=\"", end_char, sizeof(start))))
-                  {
-                     if((success = get_str(buffer, end, "end=\"", end_char, sizeof(end))))
-                     {
-                        if((success = get_str(buffer, color, "color=\"", end_char, sizeof(color))))
-                        {
-                           HighlightInfo *info;
-                           success = get_str(buffer, cclass, "class=\"", end_char, sizeof(cclass));
-                           if(success)
-                              info = get_highlight_info(color, cclass);
-                           else
-                              info = get_highlight_info(color, default_class);
-                           syntax_list = gds_editor_syntax_entry_new(syntax_list, name, start, end, info->color, NULL, info->font, info->flags, info->type);
-                           g_free(info);
-                        }
-                     }
-                  }
+                  new_table->extensions = g_list_append(new_table->extensions, (gpointer)text);
                }
             }
-         }
-      }	 
-      else if(strstr(buffer, "<pattern>"))
-      {
-         while(fgets(buffer, sizeof(buffer), ifile))
-         {
-            if(buffer[0] == '#') continue;
-            if(strstr(buffer, "</pattern>")) break;
-            else
+            else if(!strcmp(node->name, "mime-type"))
             {
-               if((success = get_str(buffer, name, "name=\"", end_char, sizeof(name))))
+               text = xmlNodeListGetString(doc, node->xmlChildrenNode, 1);
+               if(text)
                {
-                  if((success = get_str(buffer, words, "words=\"", end_char, sizeof(words))))
-                  {
-                     if((success = get_str(buffer, color, "color=\"", end_char, sizeof(color))))
-                     {
-                        HighlightInfo *info;
-                        success = get_str(buffer, cclass, "class=\"", end_char, sizeof(cclass));
-                        if(success)
-                           info = get_highlight_info(color, cclass);
-                        else
-                           info = get_highlight_info(color, default_class);
-                        pattern_list = gds_editor_pattern_entry_new(pattern_list, name, words, info->color, NULL, info->font, info->flags, info->type);
-                        g_free(info);
-                     }
-                  }
+                  new_table->mime_types = g_list_append(new_table->mime_types, (gpointer)text);
                }
             }
          }
       }
-      else if(strstr(buffer, "<embedded>"))
+      else if(!strcmp(section->name, "syntax-items"))
       {
-         while(fgets(buffer, sizeof(buffer), ifile))
+         for(node = section->xmlChildrenNode; node != NULL; node = node->next)
          {
-            if(buffer[0] == '#') continue;
-            if(strstr(buffer, "</embedded>")) break;
-            else
+            HighlightInfo *info;
+            gint found_count = 0;
+            gboolean class_found = FALSE;
+            for(item = node->xmlChildrenNode; item != NULL; item = item->next)
             {
-               if((success = get_str(buffer, name, "name=\"", end_char, sizeof(name))))
+               if(!strcmp(item->name, "name"))
                {
-                  if((success = get_str(buffer, start, "start=\"", end_char, sizeof(start))))
-                  {
-                     if((success = get_str(buffer, words, "words=\"", end_char, sizeof(words))))
-                     {
-                        if((success = get_str(buffer, end, "end=\"", end_char, sizeof(end))))
-                        {
-                           if((success = get_str(buffer, color, "color=\"", end_char, sizeof(color))))
-                           {
-                              HighlightInfo *info;
-                              success = get_str(buffer, cclass, "class=\"", end_char, sizeof(cclass));
-                              if(success)
-                                 info = get_highlight_info(color, cclass);
-                              else
-                                 info = get_highlight_info(color, default_class);
-                              embedded_list = gds_editor_embedded_entry_new(embedded_list, name, start, end, words, info->color, NULL, info->font, info->flags, info->type);
-                              g_free(info);
-                           }
-                        }
-                     }
-                  }
-               }	
+                  name = xmlNodeListGetString(doc, item->xmlChildrenNode, 1);
+                  found_count++;
+               }
+               else if(!strcmp(item->name, "start-regex"))
+               {
+                  start = xmlNodeListGetString(doc, item->xmlChildrenNode, 1);
+                  g_strconvescape(start);
+                  found_count++;
+               }
+               else if(!strcmp(item->name, "end-regex"))
+               {
+                  end = xmlNodeListGetString(doc, item->xmlChildrenNode, 1);
+                  g_strconvescape(end);
+                  found_count++;
+               }
+               else if(!strcmp(item->name, "style"))
+               {
+                  style = xmlNodeListGetString(doc, item->xmlChildrenNode, 1);
+                  found_count++;
+               }
+               else if(!strcmp(item->name, "class"))
+               {
+                  cclass = xmlNodeListGetString(doc, item->xmlChildrenNode, 1);
+                  class_found = TRUE;;
+               }
             }
+            if(found_count == 4)
+            {
+               if(class_found)
+                  info = get_highlight_info(style, cclass);
+               else
+                  info = get_highlight_info(style, default_class);
+               syntax_list = gds_editor_syntax_entry_new(syntax_list, name, start, end, info->color, NULL, info->font, info->flags, info->type);
+               g_free(info);
+            }
+            if(name) { g_free(name); name = NULL; }
+            if(start) { g_free(start); start = NULL; }
+            if(end) { g_free(end); end = NULL; }
+            if(style) { g_free(style); style = NULL; }
+            if(cclass) { g_free(cclass); cclass = NULL; }
          }
-      }	       
-   }
-   fclose(ifile);
-   g_snprintf(temp_file, sizeof(temp_file), "%s/." PACKAGE "/%s", getenv("HOME"), "highlight.py");
-
-   if(!(ifile = fopen(filename, "r"))) return(NULL);
-   if(!(ofile = fopen(temp_file, "w")))
-   {
-   	fclose(ifile);
-   	return(NULL);
-   }
-
-   while(fgets(buffer, sizeof(buffer), ifile) != NULL)
-   {
-      if(strstr(buffer, "<python>"))
+      }
+      else if(!strcmp(section->name, "pattern-items"))
       {
-         while(fgets(buffer, sizeof(buffer), ifile))
+         for(node = section->xmlChildrenNode; node != NULL; node = node->next)
          {
-            if(strstr(buffer, "</python>")) break;
-            fprintf(ofile, "%s", buffer);
+            HighlightInfo *info;
+            gint found_count = 0;
+            gboolean class_found = FALSE;
+            for(item = node->xmlChildrenNode; item != NULL; item = item->next)
+            {
+               if(!strcmp(item->name, "name"))
+               {
+                  name = xmlNodeListGetString(doc, item->xmlChildrenNode, 1);
+                  found_count++;
+               }
+               else if(!strcmp(item->name, "regex"))
+               {
+                  regex = xmlNodeListGetString(doc, item->xmlChildrenNode, 1);
+                  g_strconvescape(regex);
+                  found_count++;
+               }
+               else if(!strcmp(item->name, "style"))
+               {
+                  style = xmlNodeListGetString(doc, item->xmlChildrenNode, 1);
+                  found_count++;
+               }
+               else if(!strcmp(item->name, "class"))
+               {
+                  cclass = xmlNodeListGetString(doc, item->xmlChildrenNode, 1);
+                  class_found = TRUE;;
+               }
+            }
+            if(found_count == 3)
+            {
+               if(class_found)
+                  info = get_highlight_info(style, cclass);
+               else
+                  info = get_highlight_info(style, default_class);
+               pattern_list = gds_editor_pattern_entry_new(pattern_list, name, regex, info->color, NULL, info->font, info->flags, info->type);
+               g_free(info);
+            }
+            if(name) { g_free(name); name = NULL; }
+            if(regex) { g_free(regex); regex = NULL; }
+            if(style) { g_free(style); style = NULL; }
+            if(cclass) { g_free(cclass); cclass = NULL; }
+         }
+      }
+      else if(!strcmp(section->name, "embedded-items"))
+      {
+         for(node = section->xmlChildrenNode; node != NULL; node = node->next)
+         {
+            HighlightInfo *info;
+            gint found_count = 0;
+            gboolean class_found = FALSE;
+            for(item = node->xmlChildrenNode; item != NULL; item = item->next)
+            {
+               if(!strcmp(item->name, "name"))
+               {
+                  name = xmlNodeListGetString(doc, item->xmlChildrenNode, 1);
+                  found_count++;
+               }
+               else if(!strcmp(item->name, "start-regex"))
+               {
+                  start = xmlNodeListGetString(doc, item->xmlChildrenNode, 1);
+                  g_strconvescape(start);
+                  found_count++;
+               }
+               else if(!strcmp(item->name, "end-regex"))
+               {
+                  end = xmlNodeListGetString(doc, item->xmlChildrenNode, 1);
+                  g_strconvescape(end);
+                  found_count++;
+               }
+               else if(!strcmp(item->name, "regex"))
+               {
+                  regex = xmlNodeListGetString(doc, item->xmlChildrenNode, 1);
+                  g_strconvescape(regex);
+                  found_count++;
+               }
+               else if(!strcmp(item->name, "style"))
+               {
+                  style = xmlNodeListGetString(doc, item->xmlChildrenNode, 1);
+                  found_count++;
+               }
+               else if(!strcmp(item->name, "class"))
+               {
+                  cclass = xmlNodeListGetString(doc, item->xmlChildrenNode, 1);
+                  class_found = TRUE;;
+               }
+            }
+            if(found_count == 5)
+            {
+               if(class_found)
+                  info = get_highlight_info(style, cclass);
+               else
+                  info = get_highlight_info(style, default_class);
+               embedded_list = gds_editor_embedded_entry_new(embedded_list, name, start, end, regex, info->color, NULL, info->font, info->flags, info->type);
+               g_free(info);
+            }
+            if(name) { g_free(name); name = NULL; }
+            if(start) { g_free(start); start = NULL; }
+            if(end) { g_free(end); end = NULL; }
+            if(regex) { g_free(regex); regex = NULL; }
+            if(style) { g_free(style); style = NULL; }
+            if(cclass) { g_free(cclass); cclass = NULL; }
          }
       }
    }
-   fclose(ifile);
-   fclose(ofile);
-   ifile = fopen(temp_file, "r");
-   if(!ifile) return(NULL);
 
-   parsing_language = TRUE;
-
-   PyRun_SimpleFile(ifile, temp_file);
-   fclose(ifile);
+#ifdef WITH_PYTHON
+   run_python_file(filename);
+#endif
 
    if(syntax_list) new_table->syntax = gds_editor_syntax_table_new(syntax_list);
    if(pattern_list) new_table->pattern = gds_editor_pattern_table_new(pattern_list);
@@ -645,7 +821,6 @@ GdsFileHighlightTable *make_highlight_tables(gchar *filename)
    syntax_list = NULL;
    pattern_list = NULL;
    embedded_list = NULL;
-   parsing_language = FALSE;
    parse_table = NULL;
 
    highlight_tables = g_list_append(highlight_tables, (gpointer)new_table);
@@ -653,49 +828,44 @@ GdsFileHighlightTable *make_highlight_tables(gchar *filename)
    return(new_table);
 }
 
-gboolean get_str(gchar *source, gchar *dest, const gchar search[], const gchar end, gint max_chars)
+#ifdef WITH_PYTHON
+void run_python_file(gchar *filename)
 {
-   char cur_char;
-   char last_char = '\0';
-   char *ptr;
-   gint iterations = 0;
+   FILE *ifile;
+   FILE *ofile;
+   gchar temp_file[384];
+   gchar buffer[1024];
 
-   if((ptr = strstr(source, search)) == NULL)
-      return(FALSE);
-   ptr += strlen(search);
-   for(iterations = 0; iterations < max_chars; iterations++)
+   g_snprintf(temp_file, sizeof(temp_file), "%s/." PACKAGE "/%s", getenv("HOME"), "highlight.py");
+
+   if(!(ifile = fopen(filename, "r"))) return;
+   if(!(ofile = fopen(temp_file, "w")))
    {
-      cur_char = *ptr;
-      *dest = cur_char;
-      if(cur_char == '\0')
-         return(FALSE);
-      if(cur_char == end && last_char != '\\')
-      {
-         *dest = '\0';
-         break;
-      }
-      else if(cur_char == end && last_char == '\\')
-      {
-         dest--;
-         *dest = '\"';
-      }
-      else if(last_char == '\\' && cur_char == 'n')
-      {
-         dest--;
-         *dest = '\n';
-      }
-      else if(last_char == '\\' && cur_char == 't')
-      {
-         dest--;
-         *dest = '\t';
-      }
-      last_char = cur_char;
-      ptr++;
-      dest++;
+   	fclose(ifile);
+   	return;
    }
-   return(TRUE);
-}
 
+   while(fgets(buffer, sizeof(buffer), ifile) != NULL)
+   {
+      if(strstr(buffer, "<python-definition>"))
+      {
+         while(fgets(buffer, sizeof(buffer), ifile))
+         {
+            if(strstr(buffer, "</python-definition>")) break;
+            fprintf(ofile, "%s", buffer);
+         }
+      }
+   }
+   fclose(ifile);
+   fclose(ofile);
+
+   if(!(ifile = fopen(temp_file, "r"))) return;
+   parsing_language = TRUE;
+   PyRun_SimpleFile(ifile, temp_file);
+   fclose(ifile);
+   parsing_language = FALSE;
+}
+#endif
 
 void g_list_delete_all(GList *list)
 {
@@ -710,7 +880,6 @@ void g_list_delete_all(GList *list)
 }
 
 /* Returns a table of highlight info, to be g_free()'d by caller */
-
 HighlightInfo *get_highlight_info(gchar *color, gchar *cclass)
 {
    gint i;
@@ -724,7 +893,7 @@ HighlightInfo *get_highlight_info(gchar *color, gchar *cclass)
       info->type = FONT_ITALIC;
    else if(strchr(cclass, 'b'))
       info->type = FONT_BOLD;
-   if(cclass && strcmp(cclass, "d")) set = TRUE; //This means it came from getprefs.C, elsewise it will always be "d" (from parser above)
+   if(cclass && strcmp(cclass, "d")) set = TRUE; //This means it came from getprefs.c, elsewise it will always be "d" (from parser above)
 
    for(i = 0; i < EDITOR_COLORS; i++)
    {
@@ -738,8 +907,8 @@ HighlightInfo *get_highlight_info(gchar *color, gchar *cclass)
    }
 
    info->font = get_font(info->type);
-   if(strchr(cclass, 'u')) info->flags |= STYLE_UNDER;
-   if(strchr(cclass, 's')) info->flags |= STYLE_STRIKE;
+   if(strchr(cclass, 'u')) info->flags |= GTK_EXTEXT_STYLE_UNDER;
+   if(strchr(cclass, 's')) info->flags |= GTK_EXTEXT_STYLE_STRIKE;
    return(info);
 }
 
