@@ -1,8 +1,4 @@
-#ifndef WITH_REGEX
-#define WITH_REGEX
-#endif
 #include "gdseditor.h"
-#include "../gtkextext/gtkextext.h"
 
 //#define GENEROUS_PARSE 1
 #define EXTRA_PARSE 80
@@ -19,6 +15,17 @@ void check_pattern(GtkExText *text, gint start, gint end, GtkExTextProperty *pre
 void check_embedded(GtkExText *text, gint start, gint end, GtkExTextProperty *prev);
 gint get_syntax_end(GtkExText *text, gint pos, GdsEditorSyntaxEntry *table, GtkExTextMatch *m);
 void check_syntax(GtkExText *text, gint i, gint end);
+static void gds_editor_dnd_drop(GdsEditor *editor, GdkDragContext *context, gint x, gint y, GtkSelectionData *selection_data, guint info, guint time, gpointer data);
+
+enum
+{
+   TARGET_COLOR
+};
+static GtkTargetEntry drop_types[] =
+{
+   { "application/x-color", 0, TARGET_COLOR }
+};
+static gint n_drop_types = sizeof(drop_types) / sizeof(drop_types[0]);
 
 static GtkWidgetClass *parent_class = NULL;
 
@@ -62,12 +69,19 @@ GtkWidget* gds_editor_new()
    GdsEditor *gds_editor;
    GtkObject *hadj;
    hadj = gtk_adjustment_new(0, 0, 256, 20, 1, 80);
-   gds_editor = (GdsEditor *) gtk_widget_new(GDS_TYPE_EDITOR, "hadjustment", hadj, "vadjustment", NULL, NULL);
+   /*
+      Commented here for switch to new GtkExText
+      gds_editor = (GdsEditor *) gtk_widget_new(GDS_TYPE_EDITOR, "hadjustment", hadj, "vadjustment", NULL, NULL);
+   */
+   gds_editor = (GdsEditor *) gtk_widget_new(GDS_TYPE_EDITOR, NULL);
    gtk_signal_connect(GTK_OBJECT(gds_editor), "property_text_remove", GTK_SIGNAL_FUNC(property_text_remove), NULL);
    gtk_signal_connect(GTK_OBJECT(gds_editor), "property_text_insert", GTK_SIGNAL_FUNC(property_text_insert), NULL);
    gtk_signal_connect(GTK_OBJECT(gds_editor), "property_mark", GTK_SIGNAL_FUNC(property_mark), NULL);
    gtk_signal_connect(GTK_OBJECT(gds_editor), "key_press_event", GTK_SIGNAL_FUNC(key_press_cb), gds_editor);
    gtk_signal_connect(GTK_OBJECT(gds_editor), "changed", GTK_SIGNAL_FUNC(gds_editor_set_changed), 0);
+
+   gtk_drag_dest_set(GTK_WIDGET(gds_editor), (GtkDestDefaults) (GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_HIGHLIGHT | GTK_DEST_DEFAULT_DROP), drop_types, n_drop_types, GDK_ACTION_COPY);
+   gtk_signal_connect(GTK_OBJECT(gds_editor), "drag_data_received", GTK_SIGNAL_FUNC(gds_editor_dnd_drop), gds_editor);
    return GTK_WIDGET(gds_editor);
 }
 
@@ -84,7 +98,7 @@ void gds_editor_set_changed(GdsEditor *gds_editor, gpointer data)
 {
    g_return_if_fail(gds_editor != NULL);
    g_return_if_fail(GDS_IS_EDITOR(gds_editor));
-   if(!GTK_EXTEXT(gds_editor)->read_only)
+   if(GTK_EXTEXT(gds_editor)->editable)
       gds_editor->changed++;
    gtk_signal_emit_by_name(GTK_OBJECT(gds_editor), "move_to_column", 0, NULL);
 }
@@ -130,7 +144,6 @@ void gds_editor_highlight(GdsEditor *text)
 static gint key_press_cb(GtkWidget *widget, GdkEventKey *event, GdsEditor *gds_editor)
 {
    GtkExText *text;
-   GtkEditable *editable;
    gint curpos;
    gboolean handled = FALSE;
    gboolean shift_state;
@@ -142,22 +155,21 @@ static gint key_press_cb(GtkWidget *widget, GdkEventKey *event, GdsEditor *gds_e
    key = event->keyval;
    
    text = (GtkExText *)widget;
-   editable = (GtkEditable *)widget;
-   curpos = gtk_extext_get_position(editable);
+   curpos = gtk_extext_get_position(text);
 
    if(event->keyval == 'x' && control_state && gtk_extext_get_editable(text))
    {
-      gtk_editable_cut_clipboard(editable);
-      editable->has_selection = FALSE;
-      editable->selection_start_pos = -1;
-      editable->selection_end_pos = -1;
+      gtk_extext_cut_clipboard(text);
+      text->has_selection = FALSE;
+      text->selection_start_pos = -1;
+      text->selection_end_pos = -1;
       handled = TRUE;
    }
    else if(event->keyval == 'z' && control_state  && gtk_extext_get_editable(text))
    {
-      editable->has_selection = FALSE;
-      editable->selection_start_pos = -1;
-      editable->selection_end_pos = -1;
+      text->has_selection = FALSE;
+      text->selection_start_pos = -1;
+      text->selection_end_pos = -1;
       if(!gtk_extext_undo_is_empty(text))
       {
          gtk_extext_undo(text);
@@ -168,9 +180,9 @@ static gint key_press_cb(GtkWidget *widget, GdkEventKey *event, GdsEditor *gds_e
    }
    else if(event->keyval == 'r' && control_state && gtk_extext_get_editable(text))
    {
-      editable->has_selection = FALSE;
-      editable->selection_start_pos = -1;
-      editable->selection_end_pos = -1;
+      text->has_selection = FALSE;
+      text->selection_start_pos = -1;
+      text->selection_end_pos = -1;
       if(!gtk_extext_redo_is_empty(text))
       {
          gtk_extext_redo(text);
@@ -238,7 +250,7 @@ gint property_text_remove(GtkExText *text, GtkExTextProperty *prop, gint start,
    else if((prop = gtk_extext_property_nearest_backward(text, start, prop)) && GPOINTER_TO_INT(prop->user_data) == SYNTAX_TABLE)
    {
       start = prop->startpos;
-      elinedataptr = gtk_extext_get_line_by_char_pos(text, end);
+      elinedataptr = gtk_extext_get_line_by_char_pos(text, end, NULL);
       end = elinedataptr && elinedataptr->endpos > end ? elinedataptr->endpos : end;
       g_free(elinedataptr);
    }
@@ -246,16 +258,16 @@ gint property_text_remove(GtkExText *text, GtkExTextProperty *prop, gint start,
    else
    {
 #ifdef GENEROUS_PARSE
-      slinedataptr = gtk_extext_get_line_by_char_pos(text, start-EXTRA_PARSE > 0 ? start-EXTRA_PARSE : 0);
+      slinedataptr = gtk_extext_get_line_by_char_pos(text, start-EXTRA_PARSE > 0 ? start-EXTRA_PARSE : 0, NULL);
 #else
-      slinedataptr = gtk_extext_get_line_by_char_pos(text, start);
+      slinedataptr = gtk_extext_get_line_by_char_pos(text, start, NULL);
 #endif
       start = slinedataptr && slinedataptr->startpos < start ? slinedataptr->startpos : start;
       g_free(slinedataptr);
 #ifdef GENEROUS_PARSE
-      elinedataptr = gtk_extext_get_line_by_char_pos(text, end+EXTRA_PARSE < text->length ? end+EXTRA_PARSE : text->length);
+      elinedataptr = gtk_extext_get_line_by_char_pos(text, end+EXTRA_PARSE < text->length ? end+EXTRA_PARSE : text->length, NULL);
 #else
-      elinedataptr = gtk_extext_get_line_by_char_pos(text, end);
+      elinedataptr = gtk_extext_get_line_by_char_pos(text, end, NULL);
 #endif
       end = elinedataptr && elinedataptr->endpos > end ? elinedataptr->endpos : end;
       g_free(elinedataptr);
@@ -285,23 +297,23 @@ gint property_text_insert(GtkExText *text, GtkExTextProperty *prop, gint start,
    else if((prop = gtk_extext_property_nearest_backward(text, start, prop)) && GPOINTER_TO_INT(prop->user_data) == SYNTAX_TABLE)
    {
       start = prop->startpos;
-      elinedataptr = gtk_extext_get_line_by_char_pos(text, end);
+      elinedataptr = gtk_extext_get_line_by_char_pos(text, end, NULL);
       end = elinedataptr && elinedataptr->endpos > end ? elinedataptr->endpos : end;
       g_free(elinedataptr);
    }
    else
    {
 #ifdef GENEROUS_PARSE
-      slinedataptr = gtk_extext_get_line_by_char_pos(text, start-EXTRA_PARSE > 0 ? start-EXTRA_PARSE : 0);
+      slinedataptr = gtk_extext_get_line_by_char_pos(text, start-EXTRA_PARSE > 0 ? start-EXTRA_PARSE : 0, NULL);
 #else
-      slinedataptr = gtk_extext_get_line_by_char_pos(text, start);
+      slinedataptr = gtk_extext_get_line_by_char_pos(text, start, NULL);
 #endif
       start = slinedataptr && slinedataptr->startpos < start ? slinedataptr->startpos : start;
       g_free(slinedataptr);
 #ifdef GENEROUS_PARSE
-      elinedataptr = gtk_extext_get_line_by_char_pos(text, end+EXTRA_PARSE < text->length ? end+EXTRA_PARSE : text->length);
+      elinedataptr = gtk_extext_get_line_by_char_pos(text, end+EXTRA_PARSE < text->length ? end+EXTRA_PARSE : text->length, NULL);
 #else
-      elinedataptr = gtk_extext_get_line_by_char_pos(text, end);
+      elinedataptr = gtk_extext_get_line_by_char_pos(text, end, NULL);
 #endif
       end = elinedataptr && elinedataptr->endpos > end ? elinedataptr->endpos : end;
       g_free(elinedataptr);
@@ -486,10 +498,12 @@ void check_pattern(GtkExText *text, gint start, gint end, GtkExTextProperty *pre
 gint get_syntax_end(GtkExText *text, gint pos, GdsEditorSyntaxEntry *table, GtkExTextMatch *m)
 {
    int ret;
+   int nbesc;
    do {
       ret = gtk_extext_regex_search(text, m->endpos, &table->regex_end, TRUE, m);
-      if(ret < 0) return -1;
-   } while(m->endpos && GTK_EXTEXT_INDEX(text, m->endpos-2) == '\\' );
+      if(ret < 0) return(-1);
+      for(nbesc = 0; GTK_EXTEXT_INDEX(text, m->endpos-2-nbesc) == '\\'; nbesc++);
+   } while(m->endpos && nbesc&1);
    return(ret);
 }
 
@@ -686,50 +700,73 @@ void gds_editor_table_free(GdsEditorHighlightTable *table)
    g_free(table);
 }
 
-void gds_editor_insert_text(GdsEditor *editable, const gchar *new_text, gint new_text_length, gint *position)
+void gds_editor_insert_text(GdsEditor *editor, const gchar *new_text, gint new_text_length, gint *position)
 {
    GtkExText *text;
-   GtkEditable *edit;
-   g_return_if_fail(editable != NULL);
-   g_return_if_fail(GTK_IS_EDITABLE(editable));
-   text = GTK_EXTEXT(editable);
-   edit = GTK_EDITABLE(editable);
-   if(text->read_only) return;
-   gtk_editable_insert_text(edit, new_text, new_text_length, position);
+   g_return_if_fail(editor != NULL);
+   g_return_if_fail(GTK_IS_EXTEXT(editor));
+   text = GTK_EXTEXT(editor);
+   if(!text->editable) return;
+   gtk_extext_insert_text(text, new_text, new_text_length, position);
 }
 
-void gds_editor_delete_text(GdsEditor *editable, gint start_pos, gint end_pos)
+void gds_editor_delete_text(GdsEditor *editor, gint start_pos, gint end_pos)
 {
    GtkExText *text;
-   GtkEditable *edit;
-   g_return_if_fail(editable != NULL);
-   g_return_if_fail(GTK_IS_EDITABLE(editable));
-   text = GTK_EXTEXT(editable);
-   edit = GTK_EDITABLE(editable);
-   if(text->read_only) return;
-   gtk_editable_delete_text(edit, start_pos, end_pos);
+   g_return_if_fail(editor != NULL);
+   g_return_if_fail(GTK_IS_EXTEXT(editor));
+   text = GTK_EXTEXT(editor);
+   if(!text->editable) return;
+   gtk_extext_delete_text(text, start_pos, end_pos);
 }
 
-gchar *gds_editor_get_chars(GdsEditor *editable, gint start_pos, gint end_pos)
+gchar *gds_editor_get_chars(GdsEditor *editor, gint start_pos, gint end_pos)
 {
    GtkExText *text;
-   GtkEditable *edit;
    gchar *chars;
-   g_return_val_if_fail(editable != NULL, NULL);
-   g_return_val_if_fail(GTK_IS_EDITABLE(editable), NULL);
-   text = GTK_EXTEXT(editable);
-   edit = GTK_EDITABLE(editable);
-   chars = gtk_editable_get_chars(edit, start_pos, end_pos);
+   g_return_val_if_fail(editor != NULL, NULL);
+   g_return_val_if_fail(GTK_IS_EXTEXT(editor), NULL);
+   text = GTK_EXTEXT(editor);
+   chars = gtk_extext_get_chars(text, start_pos, end_pos);
    return(chars);
 }
 
 void gds_editor_set_editable(GdsEditor *editor, gboolean editable)
 {
    g_return_if_fail(editor != NULL);
-   g_return_if_fail(GTK_IS_EDITABLE(editor));
-   if(editable)
-      GTK_EXTEXT(editor)->read_only = FALSE;
-   else
-      GTK_EXTEXT(editor)->read_only = TRUE;
-   gtk_editable_set_editable(GTK_EDITABLE(editor), editable);
+   g_return_if_fail(GTK_IS_EXTEXT(editor));
+
+   gtk_extext_set_editable(GTK_EXTEXT(editor), editable);
+}
+
+static void gds_editor_dnd_drop(GdsEditor *editor, GdkDragContext *context, gint x, gint y, GtkSelectionData *selection_data, guint info, guint time, gpointer data)
+{
+   GtkExTextLineData *linedata;
+   gint offset = 0;
+
+   if(info == TARGET_COLOR)
+   {
+      guint16 *vals;
+      gchar string[] = "#000000";
+
+      if (selection_data->length < 0)
+         return;
+    
+      if ((selection_data->format != 16) || (selection_data->length != 8))
+      {
+         g_warning ("Received invalid color data\n");
+         return;
+      }
+      
+      vals = (guint16 *)selection_data->data;
+      vals[0] /= 256;
+      vals[1] /= 256;
+      vals[2] /= 256;
+      g_snprintf(string, sizeof(string), "#%02X%02X%02X", vals[0], vals[1], vals[2]);
+      linedata = gtk_extext_get_line_by_offset(GTK_EXTEXT(editor), y, NULL);
+      offset = gtk_extext_get_column_by_offset(GTK_EXTEXT(editor), linedata, x, NULL);
+      if(offset < 0) offset = 0;
+      offset += linedata->startpos;
+      gtk_extext_insert_text(GTK_EXTEXT(editor), string, strlen(string), &offset);
+   }
 }
