@@ -1,8 +1,26 @@
+/*  Glimmer - gdsfile.c
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Library General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
+
 #include <signal.h>
+
+#include <config.h>
 #include "gdsfile.h"
-#include "../gtkextext/gtkextext.h"
-#include "gdseditor.h"
 #include "gdstearaway.h"
+#include "../main.h"
+#include "../menus.h"
 #include "../signals.h"
 
 static void gds_file_init(GdsFile *gds_file);
@@ -205,7 +223,6 @@ static gint button_press_cb(GtkWidget *widget, GdkEventButton *event)
 static gint key_press_cb(GtkWidget *widget, GdkEventKey *event, GdsFile *gds_file)
 {
    GtkExText *text;
-   GtkEditable *editable;
    gint curpos;
    gboolean handled = FALSE;
    gint i = 0;
@@ -221,15 +238,21 @@ static gint key_press_cb(GtkWidget *widget, GdkEventKey *event, GdsFile *gds_fil
    key=event->keyval;
    
    text = (GtkExText *)widget;
-   editable = (GtkEditable *)widget;
    if(!gtk_extext_get_editable(text)) return(handled);
 
-   curpos = editable->current_pos;
+   curpos = text->current_pos;
    if(event->keyval == GDK_Tab || event->keyval == ' ' || (event->keyval == GDK_Delete && control_state))
    {
       abscol = gtk_extext_get_abs_column(text, curpos);
       tabstop = gtk_extext_get_next_tab_stop(text, curpos);
-      if(editable->has_selection && editable->selection_start_pos != editable->selection_end_pos)
+#ifdef WITH_PYTHON
+      if(event->keyval == GDK_Tab && control_state && file_emit_scripting_signal("anti-tab-pressed", gds_file))
+      {
+         handled = TRUE;
+      }
+      else
+#endif
+      if(text->has_selection && text->selection_start_pos != text->selection_end_pos)
       {
          gint start = 0;
          gint end = 0;
@@ -237,12 +260,12 @@ static gint key_press_cb(GtkWidget *widget, GdkEventKey *event, GdsFile *gds_fil
          gint last_line = 0;
          gint current_line = 0;
          GtkExTextLineData *lineptr = NULL;
-         start = editable->selection_start_pos;
-         end = editable->selection_end_pos;
-         lineptr = gtk_extext_get_line_by_char_pos(text, start);
+         start = text->selection_start_pos;
+         end = text->selection_end_pos;
+         lineptr = gtk_extext_get_line_by_char_pos(text, start, NULL);
          first_line = lineptr->line_number;
          g_free(lineptr);
-         lineptr = gtk_extext_get_line_by_char_pos(text, end);
+         lineptr = gtk_extext_get_line_by_char_pos(text, end, NULL);
          last_line = lineptr->line_number;
          if(end == lineptr->startpos && first_line < last_line) last_line--;
          g_free(lineptr);
@@ -250,67 +273,71 @@ static gint key_press_cb(GtkWidget *widget, GdkEventKey *event, GdsFile *gds_fil
          if(first_line == last_line)
          {
             gtk_extext_freeze(text);
-            gtk_editable_delete_text(editable, start, end);
-            editable->has_selection = FALSE;
-            editable->selection_start_pos = -1;
-            editable->selection_end_pos = -1;
+            gtk_extext_delete_text(text, start, end);
+            text->has_selection = FALSE;
+            text->selection_start_pos = -1;
+            text->selection_end_pos = -1;
             if(event->keyval == GDK_Tab)
             {
+#ifdef WITH_PYTHON
                if(!file_emit_scripting_signal("tab-pressed", gds_file))
                {
+#endif
                   spaces = make_spaces_string(tabstop);
                   if((gds_file->props.over_ride && gds_file->props.use_spaces) || (!gds_file->props.over_ride && global_spaces))
-                     gtk_editable_insert_text(editable, spaces, strlen(spaces), &start);
+                     gtk_extext_insert_text(text, spaces, strlen(spaces), &start);
                   else
-                     gtk_editable_insert_text(editable, "\t", 1, &start);
+                     gtk_extext_insert_text(text, "\t", 1, &start);
                   g_free(spaces);
+#ifdef WITH_PYTHON
                }
+#endif
             }
             else
             {
-               gtk_editable_insert_text(editable, " ", 1, &start);
+               gtk_extext_insert_text(text, " ", 1, &start);
             }
             gtk_extext_thaw(text);
             gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
             return(TRUE);
          }
          gtk_extext_freeze(text);
-         editable->has_selection = FALSE;
-         editable->selection_start_pos = -1;
-         editable->selection_end_pos = -1;
+         text->has_selection = FALSE;
+         text->selection_start_pos = -1;
+         text->selection_end_pos = -1;
          tabstop = text->default_tab_width;
          spaces = make_spaces_string(tabstop);
          while(current_line <= last_line)
          {
             gtk_extext_set_column(text, 0);
             gtk_extext_set_line(text, current_line);
-            chars = gtk_editable_get_chars(editable, editable->current_pos, editable->current_pos + text->default_tab_width);
-            if(event->keyval == GDK_Tab && (control_state || shift_state))
+            chars = gtk_extext_get_chars(text, text->current_pos, text->current_pos + text->default_tab_width);
+            if(event->keyval == GDK_Tab && control_state)
             {
                if(chars && !strcmp(chars, spaces))
-                  gtk_editable_delete_text(editable, editable->current_pos, editable->current_pos + text->default_tab_width);
+                  gtk_extext_delete_text(text, text->current_pos, text->current_pos + text->default_tab_width);
                else if(chars && chars[0] == '\t')
-                  gtk_editable_delete_text(editable, editable->current_pos, editable->current_pos + 1);
+                  gtk_extext_delete_text(text, text->current_pos, text->current_pos + 1);
             }
             else if(event->keyval == GDK_Tab)
             {
                if((gds_file->props.over_ride && gds_file->props.use_spaces) || (!gds_file->props.over_ride && global_spaces))
-                  gtk_editable_insert_text(editable, spaces, strlen(spaces), &editable->current_pos);
+                  gtk_extext_insert_text(text, spaces, strlen(spaces), &text->current_pos);
                else
-                  gtk_editable_insert_text(editable, "\t", 1, &editable->current_pos);
+                  gtk_extext_insert_text(text, "\t", 1, &text->current_pos);
             }
             else if(event->keyval == ' ' && (control_state || shift_state))
             {
                if(chars[0] == ' ')
-                  gtk_editable_delete_text(editable, editable->current_pos, editable->current_pos + 1);
+                  gtk_extext_delete_text(text, text->current_pos, text->current_pos + 1);
             }
             else if(event->keyval == GDK_Delete && control_state)
             {
-               gtk_editable_delete_text(editable, editable->current_pos, editable->current_pos + 1);
+               gtk_extext_delete_text(text, text->current_pos, text->current_pos + 1);
             }
             else if(event->keyval == ' ')
             {
-               gtk_editable_insert_text(editable, " ", 1, &editable->current_pos);
+               gtk_extext_insert_text(text, " ", 1, &text->current_pos);
             }
             g_free(chars);
             current_line++;
@@ -324,23 +351,27 @@ static gint key_press_cb(GtkWidget *widget, GdkEventKey *event, GdsFile *gds_fil
          else
             end = lineptr->endpos;
          g_free(lineptr);
-         gtk_extext_set_position(editable, end);
+         gtk_extext_set_position(text, end);
          gtk_extext_thaw(text);
-         gtk_editable_select_region(editable, start, end);
+         gtk_extext_select_region(text, start, end);
          g_free(spaces);
          handled = TRUE;
       }
       else if(event->keyval == GDK_Tab)
       {
+#ifdef WITH_PYTHON
          if(!file_emit_scripting_signal("tab-pressed", gds_file))
          {
+#endif
             spaces = make_spaces_string(tabstop);
             if((gds_file->props.over_ride && gds_file->props.use_spaces) || (!gds_file->props.over_ride && global_spaces))
-               gtk_editable_insert_text(editable, spaces, strlen(spaces), &editable->current_pos);
+               gtk_extext_insert_text(text, spaces, strlen(spaces), &text->current_pos);
             else
-               gtk_editable_insert_text(editable, "\t", 1, &editable->current_pos);
+               gtk_extext_insert_text(text, "\t", 1, &text->current_pos);
             g_free(spaces);
+#ifdef WITH_PYTHON
          }
+#endif
          handled = TRUE;
       }
    }
@@ -348,10 +379,12 @@ static gint key_press_cb(GtkWidget *widget, GdkEventKey *event, GdsFile *gds_fil
    {
       if((gds_file->props.over_ride && gds_file->props.auto_indent) || (!gds_file->props.over_ride && global_indent))
       {
+#ifdef WITH_PYTHON
          if(!file_emit_scripting_signal("enter-pressed", gds_file))
          {
+#endif
             gtk_extext_set_column(text, 0);
-            chars = gtk_editable_get_chars(editable, editable->current_pos, curpos);
+            chars = gtk_extext_get_chars(text, text->current_pos, curpos);
             if(chars)
             {
                for(i = 0; i < strlen(chars); i++)
@@ -366,15 +399,17 @@ static gint key_press_cb(GtkWidget *widget, GdkEventKey *event, GdsFile *gds_fil
                insert = g_new0(char, i+2);
                insert[0] = '\n';
                strncpy(&insert[1], chars, i);
-               gtk_editable_insert_text(editable, insert, i+1, &curpos);
+               gtk_extext_insert_text(text, insert, i+1, &curpos);
                g_free(insert);
             }
             else
             {
-               gtk_editable_insert_text(editable, "\n", 1, &curpos);
+               gtk_extext_insert_text(text, "\n", 1, &curpos);
             }
             g_free(chars);
+#ifdef WITH_PYTHON
          }
+#endif
          handled = TRUE;
       }
    }
@@ -400,13 +435,11 @@ void bracket_match_cb(GtkWidget *widget, gint pos, GdsFile *gds_file)
 {
    GdsEditor *gds_editor;
    GtkExText *text;
-   GtkEditable *editable;
    gboolean found = FALSE;
    g_return_if_fail(widget != NULL);
 
    gds_editor = GDS_EDITOR(widget);
    text = GTK_EXTEXT(widget);
-   editable = GTK_EDITABLE(widget);
 
    gtk_extext_set_pseudo_select(text, -1, -1);
 
@@ -415,7 +448,7 @@ void bracket_match_cb(GtkWidget *widget, gint pos, GdsFile *gds_file)
    else if(!gds_file->props.over_ride && !global_bracketmatch)
       return;
 
-   pos = editable->current_pos-1;
+   pos = text->current_pos-1;
    if(pos < 0) return;
    found = find_bracket_match(widget, &pos);
 
@@ -429,7 +462,6 @@ gboolean find_bracket_match(GtkWidget *widget, gint *search)
 {
    GdsEditor *gds_editor;
    GtkExText *text;
-   GtkEditable *editable;
    gchar base_char = 0;
    gchar search_char = 0;
    gchar cur_char = 0;
@@ -443,7 +475,6 @@ gboolean find_bracket_match(GtkWidget *widget, gint *search)
 
    gds_editor = GDS_EDITOR(widget);
    text = GTK_EXTEXT(widget);
-   editable = GTK_EDITABLE(widget);
    pos = *search;
    cur_char = GTK_EXTEXT_INDEX(text, pos);
 
