@@ -549,9 +549,11 @@ void gds_tear_away_bust_a_switch(GdsTearAway *tear_away)
    }
    else
    {
+      gint height = 0, width = 0;
+      gdk_window_get_size (GTK_WIDGET(tear_away)->window, &width, &height);
       gtk_widget_ref(GTK_WIDGET(tear_away));
       tear_away->new_parent = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-      gtk_container_set_border_width(GTK_CONTAINER(tear_away->new_parent), 5);
+      gtk_container_set_border_width(GTK_CONTAINER(tear_away->new_parent), 1);
       temp = GTK_WIDGET(tear_away)->parent;
       gtk_object_set_data(GTK_OBJECT(tear_away->new_parent), "return_data", (gpointer)temp);
       if(GTK_IS_NOTEBOOK(temp))
@@ -581,6 +583,7 @@ void gds_tear_away_bust_a_switch(GdsTearAway *tear_away)
       gtk_signal_connect(GTK_OBJECT(tear_away->new_parent), "delete_event", GTK_SIGNAL_FUNC(gds_tear_away_close_event), NULL);
       tear_away->torn = TRUE;
       if(tear_away->label_string) gds_tear_away_set_title(tear_away, tear_away->label_string);
+      gtk_widget_set_usize(tear_away->new_parent, width+2, height+2);
    }
 }
 
