@@ -7,6 +7,8 @@
 static void gnome_filelist_class_init(GnomeFileListClass *klass);
 static void gnome_filelist_init(GnomeFileList *file_list);
 static void gnome_filelist_destroy(GtkObject *object);
+static void dir_select_event(GtkCTree *tree, gint row, gint col, GdkEvent *event, GnomeFileList *file_list);
+static void dir_unselect_event(GtkCTree *tree, gint row, gint col, GdkEvent *event, GnomeFileList *file_list);
 static void file_select_event(GtkCTree *tree, gint row, gint col, GdkEvent *event, GnomeFileList *file_list);
 static void file_unselect_event(GtkCTree *tree, gint row, gint col, GdkEvent *event, GnomeFileList *file_list);
 static void gnome_filelist_get_dirs(GnomeFileList *file_list);
@@ -22,6 +24,7 @@ static void goto_last(GtkWidget *widget, GnomeFileList *file_list);
 static void check_ok_button_cb(GtkWidget *widget, GnomeFileList *file_list);
 gchar *get_parent_dir(const gchar *path);
 gchar *build_full_path(const gchar *path, const gchar *selection);
+static void check_goto_dir(GtkWidget *widget, GnomeFileList *file_list);
 static void check_goto(GtkWidget *widget, GnomeFileList *file_list);
 static void delete_file(GtkWidget *widget, GnomeFileList *file_list);
 static void rename_file(GtkWidget *widget, GnomeFileList *file_list);
@@ -131,7 +134,7 @@ GtkWidget *gnome_filelist_new_with_path(gchar *path)
    gtk_combo_disable_activate(GTK_COMBO(file_list->history_combo));
    gtk_box_pack_start(GTK_BOX(util_box), file_list->history_combo, TRUE, TRUE, 0);
    gtk_signal_connect(GTK_OBJECT(GTK_COMBO(file_list->history_combo)->entry), "changed", GTK_SIGNAL_FUNC(check_ok_button_cb), file_list);
-   gtk_signal_connect(GTK_OBJECT(GTK_COMBO(file_list->history_combo)->entry), "activate", GTK_SIGNAL_FUNC(check_goto), file_list);
+   gtk_signal_connect(GTK_OBJECT(GTK_COMBO(file_list->history_combo)->entry), "activate", GTK_SIGNAL_FUNC(check_goto_dir), file_list);
    gtk_widget_show(file_list->history_combo);
 
    paned = gtk_hpaned_new();
@@ -153,8 +156,8 @@ GtkWidget *gnome_filelist_new_with_path(gchar *path)
    gtk_clist_column_titles_passive(GTK_CLIST(file_list->directory_list));
    gtk_container_add(GTK_CONTAINER(scrolled_window), file_list->directory_list);
    gtk_signal_connect(GTK_OBJECT(file_list->directory_list), "key_press_event", GTK_SIGNAL_FUNC(gnome_filelist_key_press), 0);
-   gtk_signal_connect(GTK_OBJECT(file_list->directory_list), "select_row", GTK_SIGNAL_FUNC(file_select_event), file_list);
-   gtk_signal_connect(GTK_OBJECT(file_list->directory_list), "unselect_row", GTK_SIGNAL_FUNC(file_unselect_event), file_list);
+   gtk_signal_connect(GTK_OBJECT(file_list->directory_list), "select_row", GTK_SIGNAL_FUNC(dir_select_event), file_list);
+   gtk_signal_connect(GTK_OBJECT(file_list->directory_list), "unselect_row", GTK_SIGNAL_FUNC(dir_unselect_event), file_list);
    gtk_paned_add1(GTK_PANED(paned), scrolled_window);
    gtk_clist_set_column_title(GTK_CLIST(file_list->directory_list), 0, "Directories");
    gtk_widget_show(file_list->directory_list);
@@ -244,7 +247,7 @@ static void gnome_filelist_destroy(GtkObject *object)
    gtk_widget_destroy(file_list);
 }
 
-static void file_select_event(GtkCTree *tree, gint row, gint col, GdkEvent *event, GnomeFileList *file_list)
+static void dir_select_event(GtkCTree *tree, gint row, gint col, GdkEvent *event, GnomeFileList *file_list)
 {
    gchar *path;
    gchar *selected;
@@ -256,50 +259,65 @@ static void file_select_event(GtkCTree *tree, gint row, gint col, GdkEvent *even
    g_free(file_list->selected);
    file_list->selected = g_new(char, strlen(selected)+1);
    strcpy(file_list->selected, selected);
-   set_file_selection(file_list);
-   if(pixmap == file_list->folder->pixmap)
+   if(event && event->type == GDK_2BUTTON_PRESS)
    {
-      if(event && event->type == GDK_2BUTTON_PRESS)
+      if(!strcmp(file_list->selected, "."))
       {
-         if(!strcmp(file_list->selected, "."))
-         {
-            gnome_filelist_set_dir(file_list, file_list->path);
-         }
-         else if(!strcmp(file_list->selected, ".."))
-         {
-            path = get_parent_dir(file_list->path);
-            g_free(file_list->path);
-            file_list->path = path;
-            gnome_filelist_set_dir(file_list, file_list->path);
-         }
-   	 else
-   	 {
-            path = build_full_path(file_list->path, file_list->selected);
-            gnome_filelist_set_dir(file_list, path);
-            g_free(path);
-   	 }
+         gnome_filelist_set_dir(file_list, file_list->path);
       }
-      else if(event)
+      else if(!strcmp(file_list->selected, ".."))
       {
-         gtk_clist_unselect_all(GTK_CLIST(file_list->file_list));
+         path = get_parent_dir(file_list->path);
+         g_free(file_list->path);
+         file_list->path = path;
+         gnome_filelist_set_dir(file_list, file_list->path);
+      }
+      else
+      {
+         path = build_full_path(file_list->path, file_list->selected);
+         gnome_filelist_set_dir(file_list, path);
+         g_free(path);
       }
    }
-   else
+   else if(event)
    {
-      if(event && event->type == GDK_2BUTTON_PRESS)
-      {
-         set_file_selection(file_list);
-         gtk_signal_emit_by_name(GTK_OBJECT(file_list->ok_button), "clicked", NULL);
-      }
-      else if(event)
-      {
-         gtk_clist_unselect_all(GTK_CLIST(file_list->directory_list));
-      }
+      gtk_clist_unselect_all(GTK_CLIST(file_list->file_list));
    }
    if(!event || event->type != GDK_2BUTTON_PRESS)
       file_list->history_position = -1;
 }
 
+static void dir_unselect_event(GtkCTree *tree, gint row, gint col, GdkEvent *event, GnomeFileList *file_list)
+{
+   file_list->selected_row = -1;   
+   g_free(file_list->selected);
+   file_list->selected = NULL;
+}
+
+static void file_select_event(GtkCTree *tree, gint row, gint col, GdkEvent *event, GnomeFileList *file_list)
+{
+   gchar *selected;
+   GtkCTreeNode *node;
+   GdkPixmap *pixmap;
+   file_list->selected_row = row;   
+   node = gtk_ctree_node_nth(tree, row);
+   gtk_ctree_node_get_pixtext(GTK_CTREE(file_list->file_list), node, 0, &selected, NULL, &pixmap, NULL);
+   g_free(file_list->selected);
+   file_list->selected = g_new(char, strlen(selected)+1);
+   strcpy(file_list->selected, selected);
+   set_file_selection(file_list);
+   if(event && event->type == GDK_2BUTTON_PRESS)
+   {
+      set_file_selection(file_list);
+      gtk_signal_emit_by_name(GTK_OBJECT(file_list->ok_button), "clicked", NULL);
+   }
+   else if(event)
+   {
+      gtk_clist_unselect_all(GTK_CLIST(file_list->directory_list));
+   }
+   if(!event || event->type != GDK_2BUTTON_PRESS)
+      file_list->history_position = -1;
+}
 
 static void file_unselect_event(GtkCTree *tree, gint row, gint col, GdkEvent *event, GnomeFileList *file_list)
 {
@@ -414,7 +432,7 @@ gchar *gnome_filelist_get_filename(GnomeFileList *file_list)
    g_return_val_if_fail(GNOME_IS_FILELIST(file_list), NULL);
    path = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(file_list->history_combo)->entry));
    filename = gtk_entry_get_text(GTK_ENTRY(file_list->selection_entry));
-   full = g_strconcat(path, filename, NULL);
+   full = build_full_path(path, filename);
    return(full);
 }
 
@@ -448,7 +466,7 @@ gboolean gnome_filelist_set_dir(GnomeFileList *file_list, gchar path[])
       gtk_combo_set_popdown_strings(GTK_COMBO(file_list->history_combo), file_list->history);
    }
    g_free(temp);
-   set_file_selection(file_list);
+   gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(file_list->history_combo)->entry), file_list->path);
    gtk_widget_grab_focus(file_list->directory_list);
    file_list->selected_row = -1;
    return(TRUE);
@@ -703,7 +721,7 @@ gchar *build_full_path(const gchar *path, const gchar *selection)
    gchar chr;
    gint malloc_size = 0;
    gint offset = 0;
-   offset =  strlen(path) - 1;
+   offset = strlen(path) - 1;
    chr = path[offset];
    malloc_size = strlen(path) + 1;
    malloc_size += chr == '/' && offset ? 0 : 1;
@@ -717,6 +735,13 @@ gchar *build_full_path(const gchar *path, const gchar *selection)
    return(ptr);
 }
 
+static void check_goto_dir(GtkWidget *widget, GnomeFileList *file_list)
+{
+   gchar *path;
+   path = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(file_list->history_combo)->entry));
+   gnome_filelist_set_dir(file_list, path);
+}
+
 static void check_goto(GtkWidget *widget, GnomeFileList *file_list)
 {
    gchar *string, *path, *selected;
